#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish SEL

File : rf_sel.py

Brief : This script uses the redfish_utilities module to manage the SEL
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage the SEL on a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument(
    "--details", "-details", action="store_true", help="Indicates details to be shown for each log entry"
)
argget.add_argument("--clear", "-clear", action="store_true", help="Indicates if the log should be cleared")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

if args.debug:
    log_file = "rf_sel-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_sel Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=30, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    # Find the SEL
    match = False
    for container_type in [redfish_utilities.log_container.MANAGER, redfish_utilities.log_container.SYSTEM]:
        if container_type == redfish_utilities.log_container.MANAGER:
            container_ids = redfish_utilities.get_manager_ids(redfish_obj)
        else:
            container_ids = redfish_utilities.get_system_ids(redfish_obj)
        for container in container_ids:
            try:
                container, log_service_ids = redfish_utilities.get_log_service_ids(
                    redfish_obj, container_type=container_type, container_id=container
                )
                for log_service in log_service_ids:
                    log_service_resp = redfish_utilities.get_log_service(
                        redfish_obj, container_type=container_type, container_id=container, log_service_id=log_service
                    )
                    if (
                        log_service_resp.dict.get("LogEntryType") == "SEL"
                        or log_service_resp.dict["Id"].upper() == "SEL"
                    ):
                        match = True
                        break
            except Exception:
                pass
            if match:
                break
        if match:
            break
    if match:
        # Either clear the logs or get/print the logs
        if args.clear:
            # Clear log was requested
            print("Clearing the SEL...")
            response = redfish_utilities.clear_log_entries(redfish_obj, log_service=log_service_resp)
            response = redfish_utilities.poll_task_monitor(redfish_obj, response)
            redfish_utilities.verify_response(response)
        else:
            # Print log was requested
            log_entries = redfish_utilities.get_log_entries(redfish_obj, log_service=log_service_resp)
            try:
                from signal import signal, SIGPIPE, SIG_DFL

                signal(SIGPIPE, SIG_DFL)
            except Exception:
                # Windows does not support SIGPIPE; no need to modify the handling
                pass
            redfish_utilities.print_log_entries(log_entries, args.details)
    else:
        print("No SEL found")
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Accounts

File : rf_accounts.py

Brief : This script uses the redfish_utilities module to manage user accounts
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage user accounts on a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument(
    "--add", "-add", type=str, nargs=3, metavar=("name", "password", "role"), help="Adds a new user account"
)
argget.add_argument("--delete", "-delete", type=str, help="Deletes a user account with the given name")
argget.add_argument(
    "--setname",
    "-setname",
    type=str,
    nargs=2,
    metavar=("old_name", "new_name"),
    help="Sets a user account to a new name",
)
argget.add_argument(
    "--setpassword",
    "-setpassword",
    type=str,
    nargs=2,
    metavar=("name", "new_password"),
    help="Sets a user account to a new password",
)
argget.add_argument(
    "--setrole", "-setrole", type=str, nargs=2, metavar=("name", "new_role"), help="Sets a user account to a new role"
)
argget.add_argument("--enable", "-enable", type=str, help="Enables a user account with the given name")
argget.add_argument("--disable", "-disable", type=str, help="Disabled a user account with the given name")
argget.add_argument("--unlock", "-unlock", type=str, help="Unlocks a user account with the given name")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

# Always let the calls for account management block until the underlying task is complete
redfish_utilities.config.__auto_task_handling__ = True

if args.debug:
    log_file = "rf_accounts-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_accounts Trace")

# Set up the Redfish object
user_uri = None
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError as e:
    if args.setpassword is None:
        redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
        sys.exit(1)
    else:
        user_uri = e.args[1]
        if args.setpassword[0] == args.user:
            # The user is requesting to change their own password
            # Allow the request to go through directly, and log out
            exit_code = 0
            try:
                print("Changing password of user '{}'".format(args.setpassword[0]))
                redfish_utilities.modify_user(
                    redfish_obj, args.setpassword[0], new_password=args.setpassword[1], user_uri=user_uri
                )
            except Exception as e:
                if args.debug:
                    logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
                exit_code = 1
                print(e)
            finally:
                # Log out
                redfish_utilities.logout(
                    redfish_obj, ignore_error=True
                )  # Some services do not allow session logout in this condition
            sys.exit(exit_code)
        else:
            redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
            sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    print_accounts = True
    if args.add is not None:
        print("Adding new user '{}'".format(args.add[0]))
        redfish_utilities.add_user(redfish_obj, args.add[0], args.add[1], args.add[2])
        print_accounts = False
    if args.delete is not None:
        print("Deleting user '{}'".format(args.delete))
        redfish_utilities.delete_user(redfish_obj, args.delete)
        print_accounts = False
    if args.setname is not None:
        print("Changing name of user '{}' to '{}'".format(args.setname[0], args.setname[1]))
        redfish_utilities.modify_user(redfish_obj, args.setname[0], new_name=args.setname[1])
        print_accounts = False
    if args.setpassword is not None:
        print("Changing password of user '{}'".format(args.setpassword[0]))
        redfish_utilities.modify_user(redfish_obj, args.setpassword[0], new_password=args.setpassword[1])
        print_accounts = False
    if args.setrole is not None:
        print("Changing role of user '{}' to '{}'".format(args.setrole[0], args.setrole[1]))
        redfish_utilities.modify_user(redfish_obj, args.setrole[0], new_role=args.setrole[1])
        print_accounts = False
    if args.enable is not None:
        print("Enabling user '{}'".format(args.enable))
        redfish_utilities.modify_user(redfish_obj, args.enable, new_enabled=True)
        print_accounts = False
    if args.disable is not None:
        print("Disabling user '{}'".format(args.disable))
        redfish_utilities.modify_user(redfish_obj, args.disable, new_enabled=False)
        print_accounts = False
    if args.unlock is not None:
        print("Unlocking user '{}'".format(args.unlock))
        redfish_utilities.modify_user(redfish_obj, args.unlock, new_locked=False)
        print_accounts = False
    if print_accounts:
        user_list = redfish_utilities.get_users(redfish_obj)
        redfish_utilities.print_users(user_list)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Miscellaneous

File : misc.py

Brief : Miscellaneous functions with common script logic
"""


def logout(context, ignore_error=False):
    """
    Performs a logout of the service and allows for exceptions to be ignored

    Args:
        context: The Redfish client object with an open session
        ignore_error: Indicates if exceptions during logout are ignored
    """

    if context is not None:
        try:
            context.logout()
        except Exception:
            if ignore_error:
                pass
            else:
                raise
    return


def print_password_change_required_and_logout(context, args):
    """
    Common help text when handling password change required conditions

    Args:
        context: The Redfish client object with an open session
        args: The argparse object from the calling script
    """

    print("Password change required.  To set a new password, run the following:")
    print(
        "rf_accounts.py -r {} -u {} -p <old password> --setpassword {} <new password>".format(
            args.rhost, args.user, args.user
        )
    )
    logout(context, ignore_error=True)  # Some services do not allow session logout in this condition
    return



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish BIOS Settings

File : rf_bios_settings.py

Brief : This script uses the redfish_utilities module to manager BIOS settings of a system
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manager BIOS settings for a system")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--system", "-s", type=str, help="The ID of the system to manage")
argget.add_argument(
    "--attribute",
    "-a",
    type=str,
    nargs=2,
    metavar=("name", "value"),
    action="append",
    help="Sets a BIOS attribute to a new value; can be supplied multiple times to set multiple attributes",
)
argget.add_argument("--reset", "-reset", action="store_true", help="Resets BIOS to the default settings")
argget.add_argument(
    "--workaround",
    "-workaround",
    action="store_true",
    help="Indicates if workarounds should be attempted for non-conformant services",
    default=False,
)
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

if args.workaround:
    redfish_utilities.config.__workarounds__ = True

if args.debug:
    log_file = "rf_bios_settings-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_bios_settings Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.reset:
        # Reset BIOS to the default settings
        print("Resetting the BIOS settings...")
        redfish_utilities.reset_system_bios(redfish_obj, args.system)
    else:
        # Get the BIOS settings
        current_settings, future_settings = redfish_utilities.get_system_bios(redfish_obj, args.system)

        if args.attribute is not None:
            new_settings = {}
            for attribute in args.attribute:
                # Based on the current settings, determine the appropriate data type for the new setting
                new_value = attribute[1]
                if attribute[0] in current_settings:
                    if isinstance(current_settings[attribute[0]], bool):
                        # Boolean; convert from a string
                        if new_value.lower() == "true":
                            new_value = True
                        else:
                            new_value = False
                    elif isinstance(current_settings[attribute[0]], (int, float)):
                        # Integer or float; go by the user input to determine how to convert since the current value may be truncated
                        try:
                            new_value = int(new_value)
                        except Exception:
                            new_value = float(new_value)

                # Set the specified attribute to the new value
                new_settings[attribute[0]] = new_value
                print("Setting {} to {}...".format(attribute[0], attribute[1]))
            redfish_utilities.set_system_bios(redfish_obj, new_settings, args.system)
        else:
            # Print the BIOS settings
            redfish_utilities.print_system_bios(current_settings, future_settings)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Event Service

File : rf_event_service.py

Brief : This script uses the redfish_utilities module to manage the event service and event subscriptions
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage the event service on a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
info_argget = subparsers.add_parser("info", help="Displays information about the event service and subscriptions")
sub_argget = subparsers.add_parser("subscribe", help="Creates an event subscription to a specified URL")
sub_argget.add_argument(
    "--destination", "-dest", type=str, required=True, help="The URL where events are sent for the subscription"
)
sub_argget.add_argument(
    "--context",
    "-c",
    type=str,
    help="The context string for the subscription that is supplied back in the event payload",
)
sub_argget.add_argument(
    "--expand",
    "-e",
    action="store_true",
    help="Indicates if the origin of condition in the event is to be expanded",
    default=None,
)
sub_argget.add_argument("--format", "-f", type=str, help="The format of the event payloads")
sub_argget.add_argument(
    "--resourcetypes", "-rt", type=str, nargs="+", help="A list of resource types for the subscription"
)
sub_argget.add_argument("--registries", "-reg", type=str, nargs="+", help="A list of registries for the subscription")
sub_argget.add_argument(
    "--eventtypes",
    "-et",
    type=str,
    nargs="+",
    help="A list of event types for the subscription; this option has been deprecated in favor of other methods such as 'resource types' and 'registries'",
)
unsub_argget = subparsers.add_parser("unsubscribe", help="Deletes an event subscription")
unsub_argget.add_argument(
    "--id", "-i", type=str, required=True, help="The identifier of the event subscription to be deleted"
)
args = argget.parse_args()

if args.debug:
    log_file = "rf_event_service-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_event_service Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.command == "subscribe":
        response = redfish_utilities.create_event_subscription(
            redfish_obj,
            args.destination,
            format=args.format,
            client_context=args.context,
            expand=args.expand,
            resource_types=args.resourcetypes,
            registries=args.registries,
            event_types=args.eventtypes,
        )
        print("Created subscription '{}'".format(response.getheader("Location")))
    elif args.command == "unsubscribe":
        print("Deleting subscription '{}'".format(args.id))
        redfish_utilities.delete_event_subscription(redfish_obj, args.id)
    else:
        event_service = redfish_utilities.get_event_service(redfish_obj)
        redfish_utilities.print_event_service(event_service)
        print("")
        event_subscriptions = redfish_utilities.get_event_subscriptions(redfish_obj)
        redfish_utilities.print_event_subscriptions(event_subscriptions)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Systems Module

File : systems.py

Brief : This file contains the definitions and functionalities for interacting
        with the systems collection for a given Redfish service
"""

import warnings
import sys
from .collections import get_collection_ids
from .messages import verify_response
from .resets import reset_types
from . import config


class RedfishSystemNotFoundError(Exception):
    """
    Raised when a matching system cannot be found
    """

    pass


class RedfishSystemResetNotFoundError(Exception):
    """
    Raised when the reset action cannot be found
    """

    pass


class RedfishSystemBootNotFoundError(Exception):
    """
    Raised when the boot object cannot be found
    """

    pass


class RedfishSystemBiosNotFoundError(Exception):
    """
    Raised when the BIOS resource cannot be found
    """

    pass


class RedfishSystemBiosInvalidSettingsError(Exception):
    """
    Raised when the BIOS resource contains a settings object, but it's not rendered properly
    """

    pass


class RedfishSystemResetBiosNotFoundError(Exception):
    """
    Raised when the BIOS resource does not support the reset BIOS action
    """

    pass


class RedfishVirtualMediaNotFoundError(Exception):
    """
    Raised when a system does not have any virtual media available
    """

    pass


class RedfishNoAcceptableVirtualMediaError(Exception):
    """
    Raised when a system does not have virtual media available that meets criteria
    """

    pass


def get_system_ids(context):
    """
    Finds the system collection and returns all of the member's identifiers

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list of identifiers of the members of the system collection
    """

    # Get the service root to find the system collection
    service_root = context.get("/redfish/v1/")
    if "Systems" not in service_root.dict:
        # No system collection
        raise RedfishSystemNotFoundError("Service does not contain a system collection")

    # Get the system collection and iterate through its collection
    return get_collection_ids(context, service_root.dict["Systems"]["@odata.id"])


def get_system(context, system_id=None):
    """
    Finds a system matching the given identifier and returns its resource

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system

    Returns:
        The system resource
    """

    system_uri_pattern = "/redfish/v1/Systems/{}"
    avail_systems = None

    # If given an identifier, get the system directly
    if system_id is not None:
        system = context.get(system_uri_pattern.format(system_id))
    # No identifier given; see if there's exactly one member
    else:
        avail_systems = get_system_ids(context)
        if len(avail_systems) == 1:
            system = context.get(system_uri_pattern.format(avail_systems[0]))
        else:
            raise RedfishSystemNotFoundError(
                "Service does not contain exactly one system; a target system needs to be specified: {}".format(
                    ", ".join(avail_systems)
                )
            )

    # Check the response and return the system if the response is good
    if system.status == 404:
        if avail_systems is None:
            avail_systems = get_system_ids(context)
        raise RedfishSystemNotFoundError(
            "Service does not contain a system called {}; valid systems: {}".format(system_id, ", ".join(avail_systems))
        )
    verify_response(system)
    return system


def get_system_boot(context, system_id=None):
    """
    Finds a system matching the given ID and returns its Boot object

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system

    Returns:
        The Boot object from the given system
    """

    system = get_system(context, system_id)
    boot_obj = system.dict.get("Boot")

    if config.__workarounds__:
        # Try getting boot information from the settings resource
        settings_uris = ["Settings", "SD"]
        for setting_ext in settings_uris:
            system_settings = context.get(system.dict["@odata.id"] + "/" + setting_ext)
            if system_settings.status == 200:
                if "Boot" in system_settings.dict:
                    if boot_obj is None:
                        # Boot property only exists in settings, which is not expected
                        warnings.warn(
                            "System '{}' only reports boot information in the settings resource.  Contact your vendor.".format(
                                system_id
                            )
                        )
                        boot_obj = system_settings.dict["Boot"]
                    else:
                        # Check the boot override properties; these are expected to only be in the active resource
                        boot_override_props = [
                            "BootSourceOverrideTarget",
                            "BootSourceOverrideEnabled",
                            "BootSourceOverrideMode",
                            "UefiTargetBootSourceOverride",
                            "BootNext",
                        ]
                        boot_ov_found = False
                        for prop in boot_override_props:
                            if prop in system_settings.dict["Boot"]:
                                # Boot override property found; copy it over
                                boot_ov_found = True
                                boot_obj[prop] = system_settings.dict["Boot"][prop]
                                if prop + "@Redfish.AllowableValues" in system_settings.dict["Boot"]:
                                    boot_obj[prop + "@Redfish.AllowableValues"] = system_settings.dict["Boot"][
                                        prop + "@Redfish.AllowableValues"
                                    ]
                        if boot_ov_found:
                            warnings.warn(
                                "System '{}' contains one or more boot override properties in the settings resource.  Contact your vendor.".format(
                                    system_id
                                )
                            )
                break

    if boot_obj is None:
        raise RedfishSystemBootNotFoundError("System '{}' does not contain the boot object".format(system.dict["Id"]))
    return boot_obj


def set_system_boot(
    context, system_id=None, ov_target=None, ov_enabled=None, ov_mode=None, ov_uefi_target=None, ov_boot_next=None
):
    """
    Finds a system matching the given ID and updates its Boot object

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system
        ov_target: The override target (BootSourceOverrideTarget)
        ov_enabled: The override enabled setting (BootSourceOverrideEnabled)
        ov_mode: The override mode setting (BootSourceOverrideMode)
        ov_uefi_target: The UEFI target for override (UefiTargetBootSourceOverride)
        ov_boot_next: The UEFI boot next for override (BootNext)

    Returns:
        The response of the PATCH
    """

    # Check that the values themselves are supported by the schema
    ov_target_values = [
        "None",
        "Pxe",
        "Floppy",
        "Cd",
        "Usb",
        "Hdd",
        "BiosSetup",
        "Utilities",
        "Diags",
        "UefiShell",
        "UefiTarget",
        "SDCard",
        "UefiHttp",
        "RemoteDrive",
        "UefiBootNext",
    ]
    if ov_target is not None:
        if ov_target not in ov_target_values:
            raise ValueError(
                "{} is not an allowable override target ({})".format(ov_target, ", ".join(ov_target_values))
            )
    ov_enabled_values = ["Disabled", "Once", "Continuous"]
    if ov_enabled is not None:
        if ov_enabled not in ov_enabled_values:
            raise ValueError(
                "{} is not an allowable override enabled ({})".format(ov_enabled, ", ".join(ov_enabled_values))
            )
    ov_mode_values = ["Legacy", "UEFI"]
    if ov_mode is not None:
        if ov_mode not in ov_mode_values:
            raise ValueError("{} is not an allowable override mode ({})".format(ov_mode, ", ".join(ov_mode_values)))

    # Locate the system
    system = get_system(context, system_id)

    # Build the payload
    payload = {"Boot": {}}
    if ov_target is not None:
        payload["Boot"]["BootSourceOverrideTarget"] = ov_target
    if ov_enabled is not None:
        payload["Boot"]["BootSourceOverrideEnabled"] = ov_enabled
    if ov_mode is not None:
        payload["Boot"]["BootSourceOverrideMode"] = ov_mode
    if ov_uefi_target is not None:
        payload["Boot"]["UefiTargetBootSourceOverride"] = ov_uefi_target
    if ov_boot_next is not None:
        payload["Boot"]["BootNext"] = ov_boot_next

    # Update the system
    headers = None
    etag = system.getheader("ETag")
    if etag is not None:
        headers = {"If-Match": etag}
    response = context.patch(system.dict["@odata.id"], body=payload, headers=headers)

    # Attempt workarounds if needed
    if config.__workarounds__ and response.status >= 400:
        # Try directing the request to the settings resource
        settings_uris = ["Settings", "SD"]
        for setting_ext in settings_uris:
            system_settings = context.get(system.dict["@odata.id"] + "/" + setting_ext)
            if system_settings.status == 200:
                headers = None
                etag = system_settings.getheader("ETag")
                if etag is not None:
                    headers = {"If-Match": etag}
                settings_response = context.patch(
                    system.dict["@odata.id"] + "/" + setting_ext, body=payload, headers=headers
                )
                if settings_response.status < 400:
                    # Workaround successful; swap out the response object to return
                    warnings.warn(
                        "System '{}' incorrectly required applying the boot override configuration to the settings resource.  Contact your vendor.".format(
                            system_id
                        )
                    )
                    response = settings_response
                    break

    verify_response(response)
    return response


def print_system_boot(boot):
    """
    Prints the contents of a Boot object

    Args:
        boot: The Boot object to print
    """

    print("")

    print("Boot Override Settings:")
    boot_properties = [
        "BootSourceOverrideTarget",
        "BootSourceOverrideEnabled",
        "BootSourceOverrideMode",
        "UefiTargetBootSourceOverride",
        "BootNext",
    ]
    boot_strings = ["Target", "Enabled", "Mode", "UEFI Target", "Boot Next"]
    for index, boot_property in enumerate(boot_properties):
        if boot_property in boot:
            out_string = "  {}: {}".format(boot_strings[index], boot[boot_property])
            allow_string = ""
            if boot_property + "@Redfish.AllowableValues" in boot:
                allow_string = "; Allowable Values: {}".format(
                    ", ".join(boot[boot_property + "@Redfish.AllowableValues"])
                )
            print(out_string + allow_string)

    print("")


def get_system_reset_info(context, system_id=None, system=None):
    """
    Finds a system matching the given ID and returns its reset info

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system
        system: Existing system resource to inspect for reset info

    Returns:
        The URI of the Reset action
        A list of parameter requirements from the Action Info
    """

    if system is None:
        system = get_system(context, system_id)

    # Check that there is a Reset action
    if "Actions" not in system.dict:
        raise RedfishSystemResetNotFoundError("System '{}' does not support the reset action".format(system.dict["Id"]))
    if "#ComputerSystem.Reset" not in system.dict["Actions"]:
        raise RedfishSystemResetNotFoundError("System '{}' does not support the reset action".format(system.dict["Id"]))

    # Extract the info about the Reset action
    reset_action = system.dict["Actions"]["#ComputerSystem.Reset"]
    reset_uri = reset_action["target"]

    if "@Redfish.ActionInfo" not in reset_action:
        # No action info; need to build this manually based on other annotations

        # Default parameter requirements
        reset_parameters = [
            {"Name": "ResetType", "Required": False, "DataType": "String", "AllowableValues": reset_types}
        ]

        # Get the AllowableValues from annotations
        for param in reset_parameters:
            if param["Name"] + "@Redfish.AllowableValues" in reset_action:
                param["AllowableValues"] = reset_action[param["Name"] + "@Redfish.AllowableValues"]
    else:
        # Get the action info and its parameter listing
        action_info = context.get(reset_action["@Redfish.ActionInfo"])
        reset_parameters = action_info.dict["Parameters"]

    return reset_uri, reset_parameters


def system_reset(context, system_id=None, reset_type=None):
    """
    Finds a system matching the given ID and performs a reset

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system
        reset_type: The type of reset to perform; if None, perform one of the common resets

    Returns:
        The response of the action
    """

    # Check that the values themselves are supported by the schema
    reset_type_values = reset_types
    if reset_type is not None:
        if reset_type not in reset_type_values:
            raise ValueError("{} is not an allowable reset type ({})".format(reset_type, ", ".join(reset_type_values)))

    # Locate the reset action
    reset_uri, reset_parameters = get_system_reset_info(context, system_id)

    # Build the payload
    if reset_type is None:
        for param in reset_parameters:
            if param["Name"] == "ResetType":
                if "GracefulRestart" in param["AllowableValues"]:
                    reset_type = "GracefulRestart"
                elif "ForceRestart" in param["AllowableValues"]:
                    reset_type = "ForceRestart"
                elif "PowerCycle" in param["AllowableValues"]:
                    reset_type = "PowerCycle"

    payload = {}
    if reset_type is not None:
        payload["ResetType"] = reset_type

    # Reset the system
    response = context.post(reset_uri, body=payload)
    try:
        verify_response(response)
    except Exception as e:
        additional_message = ""
        if response.status == 400:
            # Append the list of valid reset types to 400 Bad Request responses
            additional_message = "\nNo supported reset types listed"
            for param in reset_parameters:
                if param["Name"] == "ResetType" and "AllowableValues" in param:
                    additional_message = "\nSupported reset types: {}".format(", ".join(param["AllowableValues"]))
        raise type(e)(str(e) + additional_message).with_traceback(sys.exc_info()[2])
    return response


def get_virtual_media(context, system_id=None):
    """
    Finds the system matching the given ID and gets its virtual media

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system

    Returns:
        An array of dictionaries of the virtual media instances
    """

    # Get the virtual media collection
    virtual_media_collection = get_virtual_media_collection(context, system_id=system_id)

    # Iterate through the members and pull out each of the instances
    virtual_media_list = []
    for member in virtual_media_collection.dict["Members"]:
        virtual_media = context.get(member["@odata.id"])
        virtual_media_list.append(virtual_media.dict)
    return virtual_media_list


def print_virtual_media(virtual_media_list):
    """
    Prints the virtual media list into a table

    Args:
        virtual_media_list: The virtual media list to print
    """

    virtual_media_line_format = "  {:20s} | {}: {}"
    virtual_media_properties = ["Image", "MediaTypes", "ConnectedVia", "Inserted", "WriteProtected"]
    print("")
    for virtual_media in virtual_media_list:
        print(virtual_media_line_format.format(virtual_media["Id"], "ImageName", virtual_media.get("ImageName", "")))
        for property in virtual_media_properties:
            if property in virtual_media:
                prop_val = virtual_media[property]
                if isinstance(prop_val, list):
                    prop_val = ", ".join([i for i in prop_val if i is not None])
                print(virtual_media_line_format.format("", property, prop_val))
        print("")


def insert_virtual_media(
    context, image, system_id=None, media_id=None, media_types=None, inserted=None, write_protected=None
):
    """
    Finds the system matching the given ID and inserts virtual media

    Args:
        context: The Redfish client object with an open session
        image: The URI of the media to insert
        system_id: The system to locate; if None, perform on the only system
        media_id: The virtual media instance to insert; if None, perform on an appropriate instance
        media_types: A list of acceptable media types
        inserted: Indicates if the media is to be marked as inserted for the system
        write_protected: Indicates if the media is to be marked as write-protected for the system

    Returns:
        The response of the insert operation
    """

    # Set up acceptable media types based on the image URI if not specified
    if media_types is None:
        if image.lower().endswith(".iso"):
            media_types = ["CD", "DVD"]
        elif image.lower().endswith(".img"):
            media_types = ["USBStick"]
        elif image.lower().endswith(".bin"):
            media_types = ["USBStick"]

    # Get the virtual media collection
    virtual_media_collection = get_virtual_media_collection(context, system_id=system_id)

    # Scan the virtual media for an appropriate slot
    match = False
    for member in virtual_media_collection.dict["Members"]:
        media = context.get(member["@odata.id"])
        if media.dict["Image"] is not None:
            # In use; move on
            continue

        # Check for a match
        if media_id is not None:
            if media.dict["Id"] == media_id:
                # Identifier match
                match = True
        else:
            if media_types is None:
                # No preferred media type; automatic match
                match = True
            else:
                # Check if the preferred media type is in the reported list
                for type in media_types:
                    if type in media.dict["MediaTypes"]:
                        # Acceptable media type found
                        match = True

        # If a match was found, attempt to insert the media
        if match:
            payload = {"Image": image}
            if inserted:
                payload["Inserted"] = inserted
            if write_protected:
                payload["WriteProtected"] = write_protected
            try:
                # Preference for using the InsertMedia action
                response = context.post(media.dict["Actions"]["#VirtualMedia.InsertMedia"]["target"], body=payload)
            except Exception:
                # Fallback to PATCH method
                if "Inserted" not in payload:
                    payload["Inserted"] = True
                headers = None
                etag = media.getheader("ETag")
                if etag is not None:
                    headers = {"If-Match": etag}
                response = context.patch(media.dict["@odata.id"], body=payload, headers=headers)
            verify_response(response)
            return response

    # No matches found
    if media_id is not None:
        reason = "'{}' not found or is already in use".format(media_id)
    elif media_types is not None:
        reason = "No available slots of types {}".format(", ".join(media_types))
    else:
        reason = "No available slots"
    raise RedfishNoAcceptableVirtualMediaError("No acceptable virtual media: {}".format(reason))


def eject_virtual_media(context, media_id, system_id=None):
    """
    Finds the system matching the given ID and ejects virtual media

    Args:
        context: The Redfish client object with an open session
        media_id: The virtual media instance to eject
        system_id: The system to locate; if None, perform on the only system

    Returns:
        The response of the eject operation
    """

    # Get the virtual media collection
    virtual_media_collection = get_virtual_media_collection(context, system_id=system_id)

    # Scan the virtual media for the selected slot
    for member in virtual_media_collection.dict["Members"]:
        media = context.get(member["@odata.id"])
        if media.dict["Id"] == media_id:
            # Found the selected slot; eject it
            try:
                # Preference for using the EjectMedia action
                response = context.post(media.dict["Actions"]["#VirtualMedia.EjectMedia"]["target"], body={})
            except Exception:
                # Fallback to PATCH method
                payload = {"Image": None, "Inserted": False}
                headers = None
                etag = media.getheader("ETag")
                if etag is not None:
                    headers = {"If-Match": etag}
                response = context.patch(media.dict["@odata.id"], body=payload, headers=headers)
            verify_response(response)
            return response

    # No matches found
    raise RedfishNoAcceptableVirtualMediaError("No acceptable virtual media: '{}' not found".format(media_id))


def get_virtual_media_collection(context, system_id=None):
    """
    Finds the system matching the given ID and gets its virtual media collection

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system

    Returns:
        The virtual media collection
    """

    # Locate the system
    system = get_system(context, system_id)

    # Check if there is a VirtualMediaCollection; if not, try the older location in the Manager resource
    virtual_media_uri = None
    if "VirtualMedia" in system.dict:
        virtual_media_uri = system.dict["VirtualMedia"]["@odata.id"]
    else:
        if "Links" in system.dict:
            if "ManagedBy" in system.dict["Links"]:
                for manager in system.dict["Links"]["ManagedBy"]:
                    manager_resp = context.get(manager["@odata.id"])
                    if "VirtualMedia" in manager_resp.dict:
                        # Get the first manager that contains virtual media
                        virtual_media_uri = manager_resp.dict["VirtualMedia"]["@odata.id"]
                        break
    if virtual_media_uri is None:
        raise RedfishVirtualMediaNotFoundError("System '{}' does not support virtual media".format(system.dict["Id"]))

    # Get the VirtualMediaCollection
    return context.get(virtual_media_uri)


def get_system_bios(context, system_id=None):
    """
    Finds a system matching the given ID and gets the BIOS settings

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system

    Returns:
        A dictionary of the current BIOS attributes
        A dictionary of the BIOS attributes on the next reset
    """

    # Locate the system
    system = get_system(context, system_id)

    # Get the Bios resource
    if "Bios" not in system.dict:
        raise RedfishSystemBiosNotFoundError("System '{}' does not support representing BIOS".format(system.dict["Id"]))
    bios = context.get(system.dict["Bios"]["@odata.id"])
    current_settings = bios.dict["Attributes"]
    future_settings = bios.dict["Attributes"]

    # Get the Settings object if present
    if "@Redfish.Settings" in bios.dict:
        try:
            bios_settings = get_system_bios_settings(context, bios, system.dict["Id"])
            future_settings = bios_settings.dict["Attributes"]
        except Exception:
            if config.__workarounds__:
                warnings.warn(
                    "System '{}' BIOS resource contains the settings term, but no 'SettingsObject'.  Contact your vendor.  Workarounds exhausted for reading the settings data and falling back on using the active attributes.".format(
                        system_id
                    )
                )
            else:
                raise

    return current_settings, future_settings


def set_system_bios(context, settings, system_id=None):
    """
    Finds a system matching the given ID and sets the BIOS settings

    Args:
        context: The Redfish client object with an open session
        settings: The settings to apply to the system
        system_id: The system to locate; if None, perform on the only system

    Returns:
        The response of the PATCH
    """

    # Locate the system
    system = get_system(context, system_id)

    # Get the BIOS resource and determine if the settings need to be applied to the resource itself or the settings object
    if "Bios" not in system.dict:
        raise RedfishSystemBiosNotFoundError("System '{}' does not support representing BIOS".format(system.dict["Id"]))
    bios_uri = system.dict["Bios"]["@odata.id"]
    bios = context.get(bios_uri)
    etag = bios.getheader("ETag")
    if "@Redfish.Settings" in bios.dict:
        bios_settings = get_system_bios_settings(context, bios, system.dict["Id"])
        bios_uri = bios_settings.dict["@odata.id"]
        etag = bios_settings.getheader("ETag")

    # Update the settings
    payload = {"Attributes": settings}
    headers = None
    if etag is not None:
        headers = {"If-Match": etag}
    response = context.patch(bios_uri, body=payload, headers=headers)
    verify_response(response)
    return response


def get_system_bios_settings(context, bios, system_id):
    """
    Gets the settings resource for BIOS

    Args:
        context: The Redfish client object with an open session
        bios: The BIOS resource
        system_id: The system identifier

    Returns:
        The Settings resource for BIOS
    """

    if "SettingsObject" in bios.dict["@Redfish.Settings"]:
        bios_settings = context.get(bios.dict["@Redfish.Settings"]["SettingsObject"]["@odata.id"])
    else:
        if config.__workarounds__:
            warnings.warn(
                "System '{}' BIOS resource contains the settings term, but no 'SettingsObject'.  Contact your vendor.  Attempting workarounds...".format(
                    system_id
                )
            )
            settings_uris = ["Settings", "SD"]
            for setting_ext in settings_uris:
                bios_settings = context.get(bios.dict["@odata.id"] + "/" + setting_ext)
                if bios_settings.status == 200:
                    break
            try:
                verify_response(bios_settings)
            except Exception:
                raise RedfishSystemBiosInvalidSettingsError(
                    "System '{}' BIOS resource contains the settings term, but no 'SettingsObject'.  Workarounds exhausted.  Contact your vendor.".format(
                        system_id
                    )
                ) from None
        else:
            raise RedfishSystemBiosInvalidSettingsError(
                "System '{}' BIOS resource contains the settings term, but no 'SettingsObject'.  Contact your vendor, or retry with the '__workarounds__' flag.".format(
                    system_id
                )
            )

    return bios_settings


def print_system_bios(current_settings, future_settings):
    """
    Prints the system BIOS settings into a table

    Args:
        current_settings: A dictionary of the current BIOS attributes
        future_settings: A dictionary of the BIOS attributes on the next reset
    """

    print("")
    print("BIOS Settings:")

    bios_line_format = "  {:30s} | {:30s} | {:30s}"
    print(bios_line_format.format("Attribute Name", "Current Setting", "Future Setting"))
    for attribute, value in sorted(current_settings.items()):
        if attribute in future_settings:
            print(bios_line_format.format(attribute, str(current_settings[attribute]), str(future_settings[attribute])))
        else:
            print(
                bios_line_format.format(attribute, str(current_settings[attribute]), str(current_settings[attribute]))
            )

    print("")


def reset_system_bios(context, system_id=None):
    """
    Finds a system matching the given ID and resets the BIOS settings

    Args:
        context: The Redfish client object with an open session
        system_id: The system to locate; if None, perform on the only system

    Returns:
        The response of the reset operation
    """

    # Locate the system
    system = get_system(context, system_id)

    # Get the Bios resource
    if "Bios" not in system.dict:
        raise RedfishSystemBiosNotFoundError("System '{}' does not support representing BIOS".format(system.dict["Id"]))
    bios = context.get(system.dict["Bios"]["@odata.id"])

    # Locate the ResetBios action and invoke it
    if "Actions" not in bios.dict:
        raise RedfishSystemResetBiosNotFoundError(
            "System '{}' does not support the reset BIOS action".format(system.dict["Id"])
        )
    if "#Bios.ResetBios" not in bios.dict["Actions"]:
        raise RedfishSystemResetBiosNotFoundError(
            "System '{}' does not support the reset BIOS action".format(system.dict["Id"])
        )
    response = context.post(bios.dict["Actions"]["#Bios.ResetBios"]["target"], body={})
    verify_response(response)
    return response



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Managers Module

File : managers.py

Brief : This file contains the definitions and functionalities for interacting
        with the managers collection for a given Redfish service
"""

import sys
from .collections import get_collection_ids
from .messages import verify_response
from .resets import reset_types
from .resets import reset_to_defaults_types


class RedfishManagerNotFoundError(Exception):
    """
    Raised when a matching manager cannot be found
    """

    pass


class RedfishManagerNetworkProtocolNotFoundError(Exception):
    """
    Raised when a matching manager does not contain network protocol information
    """

    pass


class RedfishManagerEthIntNotFoundError(Exception):
    """
    Raised when a matching Ethernet interface cannot be found
    """

    pass


class RedfishManagerResetNotFoundError(Exception):
    """
    Raised when the Reset action cannot be found
    """

    pass


class RedfishManagerResetToDefaultsNotFoundError(Exception):
    """
    Raised when the ResetToDefaults action cannot be found
    """

    pass


def get_manager_ids(context):
    """
    Finds the manager collection and returns all of the member's identifiers

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list of identifiers of the members of the manager collection
    """

    # Get the service root to find the manager collection
    service_root = context.get("/redfish/v1/")
    if "Managers" not in service_root.dict:
        # No manager collection
        raise RedfishManagerNotFoundError("The service does not contain a manager collection")

    # Get the manager collection and iterate through its collection
    return get_collection_ids(context, service_root.dict["Managers"]["@odata.id"])


def get_manager(context, manager_id=None):
    """
    Finds a manager matching the given identifier and returns its resource

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager

    Returns:
        The manager resource
    """

    manager_uri_pattern = "/redfish/v1/Managers/{}"
    avail_managers = None

    # If given an identifier, get the manager directly
    if manager_id is not None:
        manager = context.get(manager_uri_pattern.format(manager_id))
    # No identifier given; see if there's exactly one member
    else:
        avail_managers = get_manager_ids(context)
        if len(avail_managers) == 1:
            manager = context.get(manager_uri_pattern.format(avail_managers[0]))
        else:
            raise RedfishManagerNotFoundError(
                "The service does not contain exactly one manager; a target manager needs to be specified: {}".format(
                    ", ".join(avail_managers)
                )
            )

    # Check the response and return the manager if the response is good
    if manager.status == 404:
        if avail_managers is None:
            avail_managers = get_manager_ids(context)
        raise RedfishManagerNotFoundError(
            "The service does not contain a manager called {}; valid managers: {}".format(
                manager_id, ", ".join(avail_managers)
            )
        )
    verify_response(manager)
    return manager


def set_manager(context, manager_id=None, date_time=None, date_time_offset=None):
    """
    Finds a manager matching the given identifier and sets one or more properties

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        date_time: The date-time value to set
        date_time_offset: The date-time offset value to set

    Returns:
        The response of the PATCH
    """

    # Locate the manager
    manager = get_manager(context, manager_id)

    # Build the payload
    payload = {}
    if date_time is not None:
        payload["DateTime"] = date_time
    if date_time_offset is not None:
        payload["DateTimeLocalOffset"] = date_time_offset

    # Update the manager
    headers = None
    etag = manager.getheader("ETag")
    if etag is not None:
        headers = {"If-Match": etag}
    response = context.patch(manager.dict["@odata.id"], body=payload, headers=headers)
    verify_response(response)
    return response


def print_manager(manager):
    """
    Prints the manager info

    Args:
        manager: The manager info to print
    """

    manager_line_format = "  {}: {}"
    manager_properties = [
        "Status",
        "ManagerType",
        "PowerState",
        "FirmwareVersion",
        "DateTime",
        "DateTimeLocalOffset",
        "LastResetTime",
        "UUID",
        "ServiceEntryPointUUID",
        "Manufacturer",
        "Model",
        "PartNumber",
        "SparePartNumber",
        "SerialNumber",
    ]
    print("Manager {} Info".format(manager.dict["Id"]))
    for property in manager_properties:
        if property in manager.dict:
            prop_val = manager.dict[property]
            if isinstance(prop_val, list):
                prop_val = ", ".join([i for i in prop_val if i is not None])
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format(prop_val.get("State", "N/A"), prop_val.get("Health", "N/A"))
            print(manager_line_format.format(property, prop_val))
    print("")


def get_manager_reset_info(context, manager_id=None, manager=None):
    """
    Finds a manager matching the given ID and returns its reset info

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        manager: Existing manager resource to inspect for reset info

    Returns:
        The URI of the Reset action
        A list of parameter requirements from the action info
    """

    if manager is None:
        manager = get_manager(context, manager_id)

    # Check that there is a Reset action
    if "Actions" not in manager.dict:
        raise RedfishManagerResetNotFoundError(
            "Manager {} does not support the Reset action".format(manager.dict["Id"])
        )
    if "#Manager.Reset" not in manager.dict["Actions"]:
        raise RedfishManagerResetNotFoundError(
            "Manager {} does not support the Reset action".format(manager.dict["Id"])
        )

    # Extract the info about the Reset action
    reset_action = manager.dict["Actions"]["#Manager.Reset"]
    reset_uri = reset_action["target"]

    if "@Redfish.ActionInfo" not in reset_action:
        # No action info; need to build this manually based on other annotations

        # Default parameter requirements
        reset_parameters = [
            {"Name": "ResetType", "Required": False, "DataType": "String", "AllowableValues": reset_types}
        ]

        # Get the AllowableValues from annotations
        for param in reset_parameters:
            if param["Name"] + "@Redfish.AllowableValues" in reset_action:
                param["AllowableValues"] = reset_action[param["Name"] + "@Redfish.AllowableValues"]
    else:
        # Get the action info and its parameter listing
        action_info = context.get(reset_action["@Redfish.ActionInfo"])
        reset_parameters = action_info.dict["Parameters"]

    return reset_uri, reset_parameters


def manager_reset(context, manager_id=None, reset_type=None):
    """
    Finds a manager matching the given ID and performs a reset

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        reset_type: The type of reset to perform; if None, perform one of the common resets

    Returns:
        The response of the action
    """

    # Check that the values themselves are supported by the schema
    reset_type_values = reset_types
    if reset_type is not None:
        if reset_type not in reset_type_values:
            raise ValueError("{} is not an allowable reset type ({})".format(reset_type, ", ".join(reset_type_values)))

    # Locate the reset action
    reset_uri, reset_parameters = get_manager_reset_info(context, manager_id)

    # Build the payload
    if reset_type is None:
        for param in reset_parameters:
            if param["Name"] == "ResetType":
                if "GracefulRestart" in param["AllowableValues"]:
                    reset_type = "GracefulRestart"
                elif "ForceRestart" in param["AllowableValues"]:
                    reset_type = "ForceRestart"
                elif "PowerCycle" in param["AllowableValues"]:
                    reset_type = "PowerCycle"

    payload = {}
    if reset_type is not None:
        payload["ResetType"] = reset_type

    # Reset the manager
    response = context.post(reset_uri, body=payload)
    try:
        verify_response(response)
    except Exception as e:
        additional_message = ""
        if response.status == 400:
            # Append the list of valid reset types to 400 Bad Request responses
            additional_message = "\nNo supported reset types listed"
            for param in reset_parameters:
                if param["Name"] == "ResetType" and "AllowableValues" in param:
                    additional_message = "\nSupported reset types: {}".format(", ".join(param["AllowableValues"]))
        raise type(e)(str(e) + additional_message).with_traceback(sys.exc_info()[2])
    return response


def get_manager_reset_to_defaults_info(context, manager_id=None, manager=None):
    """
    Finds a manager matching the given ID and returns its reset-to-defaults info

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        manager: Existing manager resource to inspect for reset info

    Returns:
        The URI of the Reset action
        A list of parameter requirements from the action info
    """

    if manager is None:
        manager = get_manager(context, manager_id)

    # Check that there is a Reset action
    if "Actions" not in manager.dict:
        raise RedfishManagerResetToDefaultsNotFoundError(
            "Manager {} does not support the ResetToDefaults action".format(manager.dict["Id"])
        )
    if "#Manager.ResetToDefaults" not in manager.dict["Actions"]:
        raise RedfishManagerResetToDefaultsNotFoundError(
            "Manager {} does not support the ResetToDefaults action".format(manager.dict["Id"])
        )

    # Extract the info about the ResetToDefaults action
    reset_action = manager.dict["Actions"]["#Manager.ResetToDefaults"]
    reset_uri = reset_action["target"]

    if "@Redfish.ActionInfo" not in reset_action:
        # No action info; need to build this manually based on other annotations

        # Default parameter requirements
        reset_parameters = [
            {"Name": "ResetType", "Required": True, "DataType": "String", "AllowableValues": reset_to_defaults_types}
        ]

        # Get the AllowableValues from annotations
        for param in reset_parameters:
            if param["Name"] + "@Redfish.AllowableValues" in reset_action:
                param["AllowableValues"] = reset_action[param["Name"] + "@Redfish.AllowableValues"]
    else:
        # Get the action info and its parameter listing
        action_info = context.get(reset_action["@Redfish.ActionInfo"])
        reset_parameters = action_info.dict["Parameters"]

    return reset_uri, reset_parameters


def manager_reset_to_defaults(context, manager_id=None, reset_type=None):
    """
    Finds a manager matching the given ID and performs a reset-to-defaults

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        reset_type: The type of reset to perform; if None, perform one of the common resets

    Returns:
        The response of the action
    """

    # Check that the values themselves are supported by the schema
    reset_to_default_type_values = reset_to_defaults_types
    if reset_type is not None:
        if reset_type not in reset_to_default_type_values:
            raise ValueError(
                "{} is not an allowable reset type ({})".format(reset_type, ", ".join(reset_to_default_type_values))
            )

    # Locate the reset action
    reset_uri, reset_parameters = get_manager_reset_to_defaults_info(context, manager_id)

    # Build the payload
    if reset_type is None:
        for param in reset_parameters:
            if param["Name"] == "ResetType":
                if "PreserveNetworkAndUsers" in param["AllowableValues"]:
                    reset_type = "PreserveNetworkAndUsers"
                elif "PreserveNetwork" in param["AllowableValues"]:
                    reset_type = "PreserveNetwork"
                elif "ResetAll" in param["AllowableValues"]:
                    reset_type = "ResetAll"

    payload = {}
    if reset_type is not None:
        payload["ResetType"] = reset_type

    # Reset the manager to defaults
    response = context.post(reset_uri, body=payload)
    try:
        verify_response(response)
    except Exception as e:
        additional_message = ""
        if response.status == 400:
            # Append the list of valid reset types to 400 Bad Request responses
            additional_message = "\nNo supported reset types listed"
            for param in reset_parameters:
                if param["Name"] == "ResetType" and "AllowableValues" in param:
                    additional_message = "\nSupported reset types: {}".format(", ".join(param["AllowableValues"]))
        raise type(e)(str(e) + additional_message).with_traceback(sys.exc_info()[2])
    return response


def get_manager_network_protocol(context, manager_id=None):
    """
    Finds the network protocol information for a manager and returns its resource

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager

    Returns:
        The ManagerNetworkProtocol resource
    """
    # Get the manager to find its network protocol information
    manager = get_manager(context, manager_id)
    if "NetworkProtocol" not in manager.dict:
        # No network protocol information
        raise RedfishManagerNetworkProtocolNotFoundError(
            "Manager {} does not contain network protocol information".format(manager.dict["Id"])
        )

    # Get the network protocol information
    response = context.get(manager.dict["NetworkProtocol"]["@odata.id"])
    verify_response(response)
    return response


def set_manager_network_protocol(context, manager_id=None, network_protocol=None):
    """
    Sets network protocol settings for a manager

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        network_protocol: The network protocol settings to apply

    Returns:
        The response of the PATCH
    """
    # Get the manager to find its network protocol information
    manager = get_manager(context, manager_id)
    if "NetworkProtocol" not in manager.dict:
        # No network protocol information
        raise RedfishManagerNetworkProtocolNotFoundError(
            "Manager {} does not contain network protocol information".format(manager.dict["Id"])
        )

    # Get the current network protocol information
    response = context.get(manager.dict["NetworkProtocol"]["@odata.id"])
    verify_response(response)
    headers = None
    etag = response.getheader("ETag")
    if etag is not None:
        headers = {"If-Match": etag}

    # Set the network protocol information
    if network_protocol is None:
        network_protocol = {}
    response = context.patch(manager.dict["NetworkProtocol"]["@odata.id"], body=network_protocol, headers=headers)
    verify_response(response)
    return response


def print_manager_network_protocol(network_protocol):
    """
    Prints the manager network protocol information

    Args:
        network_protocol: The manager network protocol information to print
    """

    network_protocol_properties = [
        "HTTP",
        "HTTPS",
        "SSDP",
        "SSH",
        "Telnet",
        "KVMIP",
        "NTP",
        "RDP",
        "RFB",
        "VirtualMedia",
        "IPMI",
        "SNMP",
        "DHCP",
        "DHCPv6",
    ]
    network_protocol_line_format = "  {:16s} | {:8s} | {:6s} | {}"
    print("Manager Network Protocol Info")
    print("")
    print(network_protocol_line_format.format("Protocol", "Enabled", "Port", "Other Settings"))

    for property in network_protocol_properties:
        if property in network_protocol.dict:
            other_str = ""
            if property == "SSDP":
                # For SSDP, extract the NOTIFY settings
                other_str = []
                if "NotifyIPv6Scope" in network_protocol.dict[property]:
                    other_str.append("NOTIFY IPv6 Scope: {}".format(network_protocol.dict[property]["NotifyIPv6Scope"]))
                if "NotifyTTL" in network_protocol.dict[property]:
                    other_str.append("NOTIFY TTL: {}".format(network_protocol.dict[property]["NotifyTTL"]))
                if "NotifyMulticastIntervalSeconds" in network_protocol.dict[property]:
                    other_str.append(
                        "NOTIFY ALIVE Interval: {}".format(
                            network_protocol.dict[property]["NotifyMulticastIntervalSeconds"]
                        )
                    )
                other_str = ", ".join(other_str)
            if property == "NTP":
                # For NTP, extract the servers; need to skip "empty" slots potentially
                if "NTPServers" in network_protocol.dict[property]:
                    other_str = "NTP Servers: " + ", ".join(
                        [i for i in network_protocol.dict[property]["NTPServers"] if i is not None]
                    )
            print(
                network_protocol_line_format.format(
                    property,
                    str(network_protocol.dict[property].get("ProtocolEnabled", "")),
                    str(network_protocol.dict[property].get("Port", "")),
                    other_str,
                )
            )


def get_manager_ethernet_interface_ids(context, manager_id=None):
    """
    Finds the Ethernet interface collection for a manager and returns all of the member's identifiers

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager

    Returns:
        A list of identifiers of the members of the Ethernet interface collection
    """

    # Get the manager to find its Ethernet interface collection
    manager = get_manager(context, manager_id)
    if "EthernetInterfaces" not in manager.dict:
        # No Ethernet interface collection
        raise RedfishManagerEthIntNotFoundError(
            "Manager {} does not contain an Ethernet interface collection".format(manager.dict["Id"])
        )

    # Get the Ethernet interface collection and iterate through its collection
    return get_collection_ids(context, manager.dict["EthernetInterfaces"]["@odata.id"])


def get_manager_ethernet_interface(context, manager_id=None, interface_id=None):
    """
    Finds an Ethernet interface for a manager matching the given identifiers and returns its resource

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        interface_id: The Ethernet interface to locate; if None, perform on the only Ethernet interface

    Returns:
        The EthernetInterface resource for the manager
    """

    interface_uri_pattern = "/redfish/v1/Managers/{}/EthernetInterfaces/{}"
    avail_interfaces = None

    # Get the manager identifier in order to build the full URI later
    if manager_id is None:
        manager = get_manager(context, None)
        manager_id = manager.dict["Id"]

    # If given an identifier, get the Ethernet interface directly
    if interface_id is not None:
        interface = context.get(interface_uri_pattern.format(manager_id, interface_id))
    # No identifier given; see if there's exactly one member
    else:
        avail_interfaces = get_manager_ethernet_interface_ids(context, manager_id)
        if len(avail_interfaces) == 1:
            interface = context.get(interface_uri_pattern.format(manager_id, avail_interfaces[0]))
        else:
            raise RedfishManagerEthIntNotFoundError(
                "Manager {} does not contain exactly one Ethernet interface; a target Ethernet interface needs to be specified: {}".format(
                    manager_id, ", ".join(avail_interfaces)
                )
            )

    # Check the response and return the Ethernet interface if the response is good
    if interface.status == 404:
        if avail_interfaces is None:
            avail_interfaces = get_manager_ethernet_interface_ids(context, manager_id)
        raise RedfishManagerEthIntNotFoundError(
            "Manager {} does not contain an Ethernet interface called {}; valid Ethernet interfaces: {}".format(
                manager_id, interface_id, ", ".join(avail_interfaces)
            )
        )
    verify_response(interface)
    return interface


def set_manager_ethernet_interface(
    context,
    manager_id=None,
    interface_id=None,
    vlan=None,
    ipv4_addresses=None,
    dhcpv4=None,
    ipv6_addresses=None,
    ipv6_gateways=None,
    dhcpv6=None,
):
    """
    Finds an Ethernet interface matching the given ID and updates specified properties

    Args:
        context: The Redfish client object with an open session
        manager_id: The manager to locate; if None, perform on the only manager
        interface_id: The Ethernet interface to locate; if None, perform on the only interface
        vlan: An object containing new VLAN settings for the interface
        ipv4_addresses: An array of objects containing new IPv4 static addresses for the interface
        dhcpv4: An object containing new DHCPv4 settings for the interface
        ipv6_addresses: An array of objects containing new IPv6 static addresses for the interface
        ipv6_gateways: New IPv6 static gateways for the interface
        dhcpv6: An object containing new DHCPv6 settings for the interface

    Returns:
        The response of the PATCH
    """

    # Locate the interface
    interface = get_manager_ethernet_interface(context, manager_id, interface_id)

    # Update the interface based on the request parameters
    payload = {}
    if vlan is not None:
        payload["VLAN"] = vlan
    if ipv4_addresses is not None:
        if "IPv4StaticAddresses" in interface.dict:
            payload["IPv4StaticAddresses"] = ipv4_addresses
        else:
            # Older services manage static addresses through the "assigned addresses" property
            payload["IPv4Addresses"] = ipv4_addresses
    if dhcpv4 is not None:
        payload["DHCPv4"] = dhcpv4
    if ipv6_addresses is not None:
        payload["IPv6StaticAddresses"] = ipv6_addresses
    if ipv6_gateways is not None:
        payload["IPv6StaticDefaultGateways"] = ipv6_gateways
    if dhcpv6 is not None:
        payload["DHCPv6"] = dhcpv6
    headers = None
    etag = interface.getheader("ETag")
    if etag is not None:
        headers = {"If-Match": etag}
    response = context.patch(interface.dict["@odata.id"], body=payload, headers=headers)
    verify_response(response)
    return response


def print_manager_ethernet_interface(interface):
    """
    Prints the Ethernet interface info

    Args:
        interface: The Ethernet interface info to print
    """

    interface_line_format = "  {}: {}"
    interface_properties = [
        "Status",
        "InterfaceEnabled",
        "LinkStatus",
        "MACAddress",
        "PermanentMACAddress",
        "SpeedMbps",
        "AutoNeg",
        "FullDuplex",
        "MTUSize",
        "HostName",
        "FQDN",
        "NameServers",
        "StaticNameServers",
    ]
    print("Ethernet Interface {} Info".format(interface.dict["Id"]))
    for property in interface_properties:
        if property in interface.dict:
            prop_val = interface.dict[property]
            if isinstance(prop_val, list):
                prop_val = ", ".join([i for i in prop_val if i is not None])
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format(prop_val.get("State", "N/A"), prop_val.get("Health", "N/A"))
            print(interface_line_format.format(property, prop_val))
    print("")

    if "VLAN" in interface.dict:
        print("  VLAN Info")
        print("    Enabled: {}".format(interface.dict["VLAN"].get("VLANEnable", False)))
        print("    ID: {}".format(interface.dict["VLAN"].get("VLANId", "N/A")))
        print("    Priority: {}".format(interface.dict["VLAN"].get("VLANPriority", "N/A")))
        print("")

    print("  IPv4 Info")
    if "DHCPv4" in interface.dict:
        print("    DHCP Enabled: {}".format(interface.dict["DHCPv4"]["DHCPEnabled"]))
    if "IPv4Addresses" in interface.dict:
        print("    Assigned Addresses")
        for i, address in enumerate(interface.dict["IPv4Addresses"]):
            if address is None:
                print("      Empty")
            elif i == 0:
                print(
                    "      {}: {}, {}, {}".format(
                        address.get("Address", "N/A"),
                        address.get("SubnetMask", "N/A"),
                        address.get("Gateway", "N/A"),
                        address.get("AddressOrigin", "N/A"),
                    )
                )
            else:
                print(
                    "      {}: {}, {}".format(
                        address.get("Address", "N/A"),
                        address.get("SubnetMask", "N/A"),
                        address.get("AddressOrigin", "N/A"),
                    )
                )
    if "IPv4StaticAddresses" in interface.dict:
        print("    Static Addresses")
        for i, address in enumerate(interface.dict["IPv4StaticAddresses"]):
            if address is None:
                print("       Empty")
            elif i == 0:
                print(
                    "      {}: {}, {}".format(
                        address.get("Address", "N/A"), address.get("SubnetMask", "N/A"), address.get("Gateway", "N/A")
                    )
                )
            else:
                print("      {}: {}".format(address["Address"], address["SubnetMask"]))
    print("")

    print("  IPv6 Info")
    if "DHCPv6" in interface.dict:
        print("    DHCP Mode: {}".format(interface.dict["DHCPv6"]["OperatingMode"]))
    if "IPv6Addresses" in interface.dict:
        print("    Assigned Addresses")
        for address in interface.dict["IPv6Addresses"]:
            print(
                "      {}/{}: {}, {}".format(
                    address.get("Address", "N/A"),
                    address.get("PrefixLength", "N/A"),
                    address.get("AddressOrigin", "N/A"),
                    address.get("AddressState", "N/A"),
                )
            )
    if "IPv6StaticAddresses" in interface.dict:
        print("    Static Addresses")
        for address in interface.dict["IPv6StaticAddresses"]:
            if address is None:
                print("      Empty")
            else:
                print("      {}/{}".format(address.get("Address", "N/A"), address.get("PrefixLength", "N/A")))
    if "IPv6StaticDefaultGateways" in interface.dict:
        print("    Static Default Gateways")
        for address in interface.dict["IPv6StaticDefaultGateways"]:
            if address is None:
                print("      Empty")
            else:
                print("      {}/{}".format(address.get("Address", "N/A"), address.get("PrefixLength", "N/A")))
    if "IPv6DefaultGateway" in interface.dict:
        print("    Default Gateway: {}".format(interface.dict["IPv6DefaultGateway"]))
    if "IPv6AddressPolicyTable" in interface.dict:
        print("    Address Policy Table")
        for policy in interface.dict["IPv6AddressPolicyTable"]:
            if policy is None:
                print("      Empty")
            else:
                print(
                    "      Prefix: {}, Prec: {}, Label: {}".format(
                        policy.get("Prefix", "N/A"), policy.get("Precedence", "N/A"), policy.get("Label", "N/A")
                    )
                )
    print("")



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Thermal Equipment Module

File : thermal_equipment.py

Brief : This file contains the definitions and functionalities for interacting
        with the thermal equipment for a given Redfish service
"""

from .collections import get_collection_ids
from .messages import verify_response
from enum import Enum


class RedfishThermalEquipmentNotFoundError(Exception):
    """
    Raised when the requested thermal equipment is not represented in the service
    """

    pass


class RedfishThermalEquipmentComponentNotFoundError(Exception):
    """
    Raised when the requested thermal equipment does not contain the desired component
    """

    pass


class thermal_equipment_types(Enum):
    """
    Types of thermal equipment that can be managed
    """

    CDU = "CDU"
    HEAT_EXCHANGER = "HeatExchanger"
    IMMERSION_UNIT = "ImmersionUnit"

    def __str__(self):
        return self.value

    @property
    def plural(self):
        return self.value + "s"


class thermal_equipment_component_types(Enum):
    """
    Types of components found on thermal equipment
    """

    PRIMARY_CONNECTOR = "PrimaryCoolantConnector"
    SECONDARY_CONNECTOR = "SecondaryCoolantConnector"
    PUMP = "Pump"
    FILTER = "Filter"
    RESERVOIR = "Reservoir"

    def __str__(self):
        return self.value

    @property
    def plural(self):
        return self.value + "s"

    @property
    def short_name(self):
        if self.value == "PrimaryCoolantConnector":
            return "Primary"
        if self.value == "SecondaryCoolantConnector":
            return "Secondary"
        return self.value


def get_thermal_equipment_ids(context):
    """
    Finds the thermal equipment and returns all thermal equipment identifiers

    Args:
        context: The Redfish client object with an open session

    Returns:
        A dictionary containing lists of identifiers for each type of thermal equipment
    """

    # Get the service root to find the power equipment
    service_root = context.get("/redfish/v1/")
    if "ThermalEquipment" not in service_root.dict:
        # No thermal equipment
        raise RedfishThermalEquipmentNotFoundError("The service does not contain any thermal equipment")

    # Get the thermal equipment sets
    thermal_equipment = context.get(service_root.dict["ThermalEquipment"]["@odata.id"])
    verify_response(thermal_equipment)

    # Build up the lists of identifiers from the thermal equipment
    thermal_equipment_identifiers = {}
    for equip_type in thermal_equipment_types:
        thermal_equipment_identifiers[equip_type.plural] = []
        if equip_type.plural in thermal_equipment.dict:
            # Get the collection identifiers for this thermal equipment type
            thermal_equipment_identifiers[equip_type.plural] = get_collection_ids(
                context, thermal_equipment.dict[equip_type.plural]["@odata.id"]
            )

    return thermal_equipment_identifiers


def get_thermal_equipment_summary(context):
    """
    Gets a summary of all thermal equipment

    Args:
        context: The Redfish client object with an open session

    Returns:
        A dictionary containing lists of summary info for each type of thermal equipment
    """

    # Get the equipment identifiers
    thermal_equipment_identifiers = get_thermal_equipment_ids(context)

    # Build up the lists of summary info from the discovered identifiers
    thermal_equipment_summary = {}
    for equipment_type in thermal_equipment_types:
        thermal_equipment_summary[equipment_type.plural] = []
        if equipment_type.plural in thermal_equipment_identifiers:
            for equip in thermal_equipment_identifiers[equipment_type.plural]:
                # Get the equipment info
                equipment = get_thermal_equipment(context, equipment_type, equip)
                summary = {
                    "Id": equipment["Info"].dict.get("Id"),
                    "Model": equipment["Info"].dict.get("Model"),
                    "SerialNumber": equipment["Info"].dict.get("SerialNumber"),
                    "State": equipment["Info"].dict.get("Status", {}).get("State"),
                    "Health": equipment["Info"].dict.get("Status", {}).get("Health"),
                }
                thermal_equipment_summary[equipment_type.plural].append(summary)

    return thermal_equipment_summary


def print_thermal_equipment_summary(thermal_equipment_summary):
    """
    Prints the thermal equipment summary into a table

    Args:
        thermal_equipment_summary: The thermal equipment summary to print
    """

    summary_line_format = "  {:16s} | {:16s} | {:16s} | {:12s} | {:8s}"

    for equipment_type in thermal_equipment_summary:
        if len(thermal_equipment_summary[equipment_type]):
            print(equipment_type)
            print(summary_line_format.format("Id", "Model", "Serial Number", "State", "Health"))
            for equipment in thermal_equipment_summary[equipment_type]:
                print(
                    summary_line_format.format(
                        equipment["Id"] if equipment["Id"] else "",
                        equipment["Model"] if equipment["Model"] else "",
                        equipment["SerialNumber"] if equipment["SerialNumber"] else "",
                        equipment["State"] if equipment["State"] else "",
                        equipment["Health"] if equipment["Health"] else "",
                    )
                )
            print("")


def get_thermal_equipment(
    context,
    thermal_equipment_type,
    thermal_equipment_id=None,
    get_metrics=False,
    get_primary_connectors=False,
    get_secondary_connectors=False,
    get_pumps=False,
    get_filters=False,
    get_reservoirs=False,
    get_leak_detection=False,
    get_leak_detectors=False,
):
    """
    Finds a therma equipment matching the given identifier and returns its resource

    Args:
        context: The Redfish client object with an open session
        thermal_equipment_type: The thermal equipment type to get; see thermal_equipment_types enumeration
        thermal_equipment_id: The thermal equipment to locate; if None, perform on the only thermal equipment
        get_metrics: Indicates if the metrics should be returned
        get_primary_connectors: Indicates if the primary connectors should be returned
        get_secondary_connectors: Indicates if the secondary connectors should be returned
        get_pumps: Indicates if the pumps should be returned
        get_filters: Indicates if the filters should be returned
        get_reservoirs: Indicates if the reservoirs should be returned
        get_leak_detection: Indicates if the leak detection info should be returned
        get_leak_detectors: Indicates if the leak detectors should be returned

    Returns:
        A dictionary containing the thermal equipment resource and optional subordinate resources
    """

    thermal_equipment_uri_pattern = "/redfish/v1/ThermalEquipment/" + thermal_equipment_type.plural + "/{}"
    avail_thermal_equipment = None
    thermal_equipment = {"Info": None}
    if get_leak_detectors:
        # Getting leak detectors relies on getting the overall leak detection
        get_leak_detection = True

    # If given an identifier, get the thermal equipment directly
    if thermal_equipment_id is not None:
        thermal_equipment["Info"] = context.get(thermal_equipment_uri_pattern.format(thermal_equipment_id))
    # No identifier given; see if there's exactly one member
    else:
        all_thermal_equipment = get_thermal_equipment_ids(context)
        if thermal_equipment_type.plural not in all_thermal_equipment:
            raise RedfishThermalEquipmentNotFoundError(
                "The service does not contain any {}".format(thermal_equipment_type.plural)
            )
        avail_thermal_equipment = all_thermal_equipment[thermal_equipment_type.plural]
        if len(avail_thermal_equipment) == 1:
            thermal_equipment["Info"] = context.get(thermal_equipment_uri_pattern.format(avail_thermal_equipment[0]))
        else:
            raise RedfishThermalEquipmentNotFoundError(
                "The service contains multiple {}; a target thermal equipment needs to be specified: {}".format(
                    thermal_equipment_type.plural, ", ".join(avail_thermal_equipment)
                )
            )

    # Check the response and return the manager if the response is good
    if thermal_equipment["Info"].status == 404:
        if avail_thermal_equipment is None:
            all_equipment = get_thermal_equipment_ids(context)
            if thermal_equipment_type.plural not in all_equipment:
                raise RedfishThermalEquipmentNotFoundError(
                    "The service does not contain any {}".format(thermal_equipment_type.plural)
                )
            avail_equipment = all_equipment[thermal_equipment_type.plural]
        raise RedfishThermalEquipmentNotFoundError(
            "The service does not contain any {} called {}; valid {}: {}".format(
                thermal_equipment_type.plural,
                thermal_equipment_id,
                thermal_equipment_type.plural,
                ", ".join(avail_equipment),
            )
        )
    verify_response(thermal_equipment["Info"])

    # Get each of the subordinate resources requested by the caller
    resource_grabber = [
        {"Property": "EnvironmentMetrics", "Parameter": get_metrics, "Collection": False, "SubResource": "Info"},
        {
            "Property": "PrimaryCoolantConnectors",
            "Parameter": get_primary_connectors,
            "Collection": True,
            "SubResource": "Info",
        },
        {
            "Property": "SecondaryCoolantConnectors",
            "Parameter": get_secondary_connectors,
            "Collection": True,
            "SubResource": "Info",
        },
        {"Property": "Pumps", "Parameter": get_pumps, "Collection": True, "SubResource": "Info"},
        {"Property": "Filters", "Parameter": get_filters, "Collection": True, "SubResource": "Info"},
        {"Property": "Reservoirs", "Parameter": get_reservoirs, "Collection": True, "SubResource": "Info"},
        {"Property": "LeakDetection", "Parameter": get_leak_detection, "Collection": False, "SubResource": "Info"},
        {
            "Property": "LeakDetectors",
            "Parameter": get_leak_detectors,
            "Collection": True,
            "SubResource": "LeakDetection",
        },
    ]
    for resource in resource_grabber:
        thermal_equipment[resource["Property"]] = None
        if resource["Parameter"] and resource["Property"] in thermal_equipment[resource["SubResource"]].dict:
            if resource["Collection"]:
                thermal_equipment[resource["Property"]] = []
                collection = get_collection_ids(
                    context, thermal_equipment[resource["SubResource"]].dict[resource["Property"]]["@odata.id"]
                )
                for member_id in collection:
                    member = context.get(
                        thermal_equipment[resource["SubResource"]].dict[resource["Property"]]["@odata.id"]
                        + "/"
                        + member_id
                    )
                    verify_response(member)
                    thermal_equipment[resource["Property"]].append(member)
            else:
                thermal_equipment[resource["Property"]] = context.get(
                    thermal_equipment[resource["SubResource"]].dict[resource["Property"]]["@odata.id"]
                )
                verify_response(thermal_equipment[resource["Property"]])

    return thermal_equipment


def print_thermal_equipment(thermal_equipment):
    """
    Prints the thermal equipment info

    Args:
        thermal_equipment: The thermal equipment info to print
    """

    thermal_equipment_line_format = "  {}: {}"
    print("{} {} Info".format(thermal_equipment["Info"].dict["EquipmentType"], thermal_equipment["Info"].dict["Id"]))

    print("")
    thermal_equipment_properties = [
        "Status",
        "EquipmentType",
        "Model",
        "Manufacturer",
        "PartNumber",
        "SerialNumber",
        "Version",
        "FirmwareVersion",
        "ProductionDate",
        "AssetTag",
        "UserLabel",
        "CoolingCapacityWatts",
        "Coolant",
    ]
    for property in thermal_equipment_properties:
        if property in thermal_equipment["Info"].dict:
            prop_val = thermal_equipment["Info"].dict[property]
            if isinstance(prop_val, list):
                prop_val = ", ".join([i for i in prop_val if i is not None])
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format(prop_val.get("State", "N/A"), prop_val.get("Health", "N/A"))
            elif property == "Coolant":
                prop_val = get_coolant_string(prop_val)
            print(thermal_equipment_line_format.format(property, prop_val))
    if thermal_equipment["EnvironmentMetrics"]:
        thermal_equipment_properties = [
            ("PowerWatts", "Power", "W"),
            ("EnergykWh", "Energy", "kWh"),
            ("TemperatureCelsius", "Temperature", "C"),
            ("HumidityPercent", "Humidity", "%"),
            ("PowerLoadPercent", "PowerLoad", "%"),
            ("AbsoluteHumidity", "AbsoluteHumidity", "g/m^3"),
        ]
        for property in thermal_equipment_properties:
            if property[0] in thermal_equipment["EnvironmentMetrics"].dict:
                if thermal_equipment["EnvironmentMetrics"].dict[property[0]]["Reading"] is None:
                    prop_val = "Unavailable"
                else:
                    prop_val = (
                        str(thermal_equipment["EnvironmentMetrics"].dict[property[0]]["Reading"]) + " " + property[2]
                    )
                print(thermal_equipment_line_format.format(property[1], prop_val))
    if thermal_equipment["LeakDetection"]:
        if "LeakDetectorGroups" in thermal_equipment["LeakDetection"].dict:
            for i, group in enumerate(thermal_equipment["LeakDetection"].dict["LeakDetectorGroups"]):
                group_name = "Leak Detection Group {}".format(i)
                if "GroupName" in group:
                    group_name = group["GroupName"]
                prop_val = group.get("Status", {}).get("Health", "OK")
                if "HumidityPercent" in group:
                    prop_val = prop_val + ", Humidity: {} %".format(
                        str(group.get("HumidityPercent", {}).get("Reading", "---"))
                    )
                print(thermal_equipment_line_format.format(group_name, prop_val))
    print("")


def get_thermal_equipment_component(
    context, thermal_equipment_type, thermal_equipment_component_type, thermal_equipment_id=None, component_id=None
):
    """
    Finds a component for an instance of thermal equipment and returns its resource

    Args:
        context: The Redfish client object with an open session
        thermal_equipment_type: The thermal equipment type to get; see thermal_equipment_types enumeration
        thermal_equipment_component_type: The component type to get; see thermal_equipment_component_types enumeration
        thermal_equipment_id: The thermal equipment to locate; if None, perform on the only thermal equipment
        component_id: The component to locate; if None, perform on the only component

    Returns:
        The component resource
    """

    # Get the thermal equipment
    thermal_equipment = get_thermal_equipment(context, thermal_equipment_type, thermal_equipment_id)

    # Check that the thermal equipment has the desired component type
    if thermal_equipment_component_type.plural not in thermal_equipment["Info"].dict:
        raise RedfishThermalEquipmentComponentNotFoundError(
            "{} {} does not contain any {}".format(
                thermal_equipment_type.plural,
                thermal_equipment["Info"].dict["Id"],
                thermal_equipment_component_type.plural,
            )
        )

    thermal_equipment_component_uri_pattern = (
        thermal_equipment["Info"].dict[thermal_equipment_component_type.plural]["@odata.id"] + "/{}"
    )
    avail_component = None

    # If given an identifier, get the outlet or circuit directly
    if component_id is not None:
        component = context.get(thermal_equipment_component_uri_pattern.format(component_id))
    # No identifier given; see if there's exactly one member
    else:
        avail_component = get_collection_ids(
            context, thermal_equipment["Info"].dict[thermal_equipment_component_type.plural]["@odata.id"]
        )
        if len(avail_component) == 1:
            component = context.get(thermal_equipment_component_uri_pattern.format(avail_component[0]))
        else:
            raise RedfishThermalEquipmentComponentNotFoundError(
                "{} {} contains multiple {}; a target component needs to be specified: {}".format(
                    thermal_equipment_type.plural,
                    thermal_equipment["Info"].dict["Id"],
                    thermal_equipment_component_type.plural,
                    ", ".join(avail_component),
                )
            )

    # Check the response and return the manager if the response is good
    if component.status == 404:
        if avail_component is None:
            avail_component = get_collection_ids(
                context, thermal_equipment["Info"].dict[thermal_equipment_component_type.plural]["@odata.id"]
            )
        raise RedfishThermalEquipmentNotFoundError(
            "{} {} does not contain any {} called {}; valid {}: {}".format(
                thermal_equipment_type.plural,
                thermal_equipment["Info"].dict["Id"],
                thermal_equipment_component_type.plural,
                component_id,
                thermal_equipment_component_type.plural,
                ", ".join(avail_component),
            )
        )
    verify_response(component)
    return component


def print_thermal_equipment_component(component):
    """
    Prints the component info

    Args:
        component: The component to print
    """

    component_line_format = "  {}: {}"

    if "/PrimaryCoolantConnectors/" in component.dict["@odata.id"]:
        print("Primary Connector {} Info".format(component.dict["Id"]))
    elif "/SecondaryCoolantConnectors/" in component.dict["@odata.id"]:
        print("Secondary Connector {} Info".format(component.dict["Id"]))
    else:
        print("{} {} Info".format(component.dict["@odata.type"].split(".")[-1], component.dict["Id"]))

    print("")
    # Display the simple properties
    component_properties = [
        "Status",
        "FluidLevelStatus",
        "CoolantConnectorType",
        "ReservoirType",
        "PumpType",
        "Model",
        "Manufacturer",
        "PartNumber",
        "SerialNumber",
        "Version",
        "FirmwareVersion",
        "ProductionDate",
        "AssetTag",
        "UserLabel",
        "Coolant",
        "RatedFlowLitersPerMinute",
        "RatedFlowPressurekPa",
        "RatedPressurekPa",
        "ServicedDate",
        "ServiceHours",
        "RatedServiceHours",
        "Replaceable",
        "HotPluggable",
        "CapacityLiters",
    ]
    for property in component_properties:
        if property in component.dict:
            prop_val = component.dict[property]
            if isinstance(prop_val, list):
                prop_val = ", ".join([i for i in prop_val if i is not None])
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format(prop_val.get("State", "N/A"), prop_val.get("Health", "N/A"))
            elif property == "Coolant":
                prop_val = prop_val = get_coolant_string(prop_val)
            elif property == "FluidLevelStatus":
                if "FluidLevelPercent" in component.dict:
                    prop_val = (
                        prop_val + " (" + str(component.dict.get("FluidLevelPercent", {}).get("Reading", "---")) + " %)"
                    )
            else:
                prop_val = str(prop_val)
            print(component_line_format.format(property, prop_val))
    # Display the embedded metrics
    component_properties = [
        ("FlowLitersPerMinute", "Flow", "L/min"),
        ("HeatRemovedkW", "HeatRemoved", "kW"),
        ("SupplyTemperatureCelsius", "SupplyTemperature", "C"),
        ("ReturnTemperatureCelsius", "ReturnTemperature", "C"),
        ("DeltaTemperatureCelsius", "DeltaTemperature", "C"),
        ("SupplyPressurekPa", "SupplyPressure", "kPa"),
        ("ReturnPressurekPa", "ReturnPressure", "kPa"),
        ("DeltaPressurekPa", "DeltaPressure", "kPa"),
        ("PumpSpeedPercent", "PumpSpeed", "%"),
    ]
    for property in component_properties:
        if property[0] in component.dict:
            if component.dict[property[0]]["Reading"] is None:
                prop_val = "Unavailable"
            else:
                prop_val = str(component.dict[property[0]]["Reading"]) + " " + property[2]
            print(component_line_format.format(property[1], prop_val))
    # Display compounded embedded metrics
    component_properties = [
        ("Temperature", "Celsius", "C"),
        ("Pressure", "kPa", "kPa"),
    ]
    for property in component_properties:
        compound_val = []
        for measurement_point in ["Supply", "Return", "Delta"]:
            prop_name = measurement_point + property[0] + property[1]
            if prop_name in component.dict:
                if component.dict[prop_name]["Reading"] is None:
                    prop_val = "Unavailable"
                else:
                    prop_val = str(component.dict[prop_name]["Reading"]) + " " + property[2]
                compound_val.append(measurement_point + " " + prop_val)
        if compound_val:
            print(component_line_format.format(property[0], ", ".join(compound_val)))
    print("")


def print_thermal_equipment_component_summary(thermal_equipment, component_type, print_heading):
    """
    Prints the component summary for an instance of thermal equipment

    Args:
        thermal_equipment: The thermal equipment info to print
        component_type: The type of component info to print
        print_heading: Indicates if the tabling heading should be printed
    """

    if not thermal_equipment[component_type.plural]:
        if print_heading:
            print(
                "{} {} does not contain any {}".format(
                    thermal_equipment["Info"].dict["EquipmentType"],
                    thermal_equipment["Info"].dict["Id"],
                    component_type.plural,
                )
            )
        return

    if print_heading:
        print(
            "{} {} {}".format(
                thermal_equipment["Info"].dict["EquipmentType"],
                thermal_equipment["Info"].dict["Id"],
                component_type.plural,
            )
        )
        print("")

    if component_type == thermal_equipment_component_types.PUMP:
        component_line_format = "  {:16s} | {:12s} | {:12s} | {:12s} | {:12s}"
        print(
            component_line_format.format(
                component_type.short_name + " Id",
                "Type",
                "State",
                "Health",
                "Speed (%)",
            )
        )
        for item in thermal_equipment[component_type.plural]:
            type = str(item.dict.get("PumpType", "---"))
            state = str(item.dict.get("Status", {}).get("State", "---"))
            health = str(item.dict.get("Status", {}).get("Health", "---"))
            speed = str(item.dict.get("PumpSpeedPercent", {}).get("Reading", "---"))
            print(component_line_format.format(item.dict["Id"], type, state, health, speed))
    elif component_type == thermal_equipment_component_types.FILTER:
        component_line_format = "  {:16s} | {:12s} | {:12s} | {:25s} | {:12s}"
        print(
            component_line_format.format(
                component_type.short_name + " Id",
                "State",
                "Health",
                "Serviced Date",
                "Service Hours",
            )
        )
        for item in thermal_equipment[component_type.plural]:
            state = str(item.dict.get("Status", {}).get("State", "---"))
            health = str(item.dict.get("Status", {}).get("Health", "---"))
            serviced_date = str(item.dict.get("ServicedDate", "---"))
            service_hours = str(item.dict.get("ServiceHours", "---"))
            print(component_line_format.format(item.dict["Id"], state, health, serviced_date, service_hours))
    elif component_type == thermal_equipment_component_types.RESERVOIR:
        component_line_format = "  {:16s} | {:12s} | {:12s} | {:12s} | {:20s}"
        print(
            component_line_format.format(
                component_type.short_name + " Id",
                "Type",
                "State",
                "Health",
                "Fluid Status",
            )
        )
        for item in thermal_equipment[component_type.plural]:
            type = str(item.dict.get("ReservoirType", "---"))
            state = str(item.dict.get("Status", {}).get("State", "---"))
            health = str(item.dict.get("Status", {}).get("Health", "---"))
            fluid_status = str(item.dict.get("FluidLevelStatus", "---"))
            if "FluidLevelPercent" in item.dict:
                fluid_status = (
                    fluid_status + " (" + str(item.dict.get("FluidLevelPercent", {}).get("Reading", "---")) + " %)"
                )
            print(component_line_format.format(item.dict["Id"], type, state, health, fluid_status))
    else:
        component_line_format = "  {:16s} | {:12s} | {:12s} | {:14s} | {:12s} | {:14s}"
        print(
            component_line_format.format(
                component_type.short_name + " Id",
                "Flow (L/min)",
                "Supply T (C)",
                "Supply P (kPa)",
                "Return T (C)",
                "Return P (kPa)",
            )
        )
        for item in thermal_equipment[component_type.plural]:
            flow = str(item.dict.get("FlowLitersPerMinute", {}).get("Reading", "---"))
            supply_temp = str(item.dict.get("SupplyTemperatureCelsius", {}).get("Reading", "---"))
            supply_pressure = str(item.dict.get("SupplyPressurekPa", {}).get("Reading", "---"))
            return_temp = str(item.dict.get("ReturnTemperatureCelsius", {}).get("Reading", "---"))
            return_pressure = str(item.dict.get("ReturnPressurekPa", {}).get("Reading", "---"))
            print(
                component_line_format.format(
                    item.dict["Id"], flow, supply_temp, supply_pressure, return_temp, return_pressure
                )
            )
    print("")


def print_thermal_equipment_leak_detector_summary(thermal_equipment, print_heading):
    """
    Prints the leak detector summary for an instance of thermal equipment

    Args:
        thermal_equipment: The thermal equipment info to print
        print_heading: Indicates if the tabling heading should be printed
    """

    if not thermal_equipment["LeakDetection"]:
        if print_heading:
            print(
                "{} {} does not contain any leak detection".format(
                    thermal_equipment["Info"].dict["EquipmentType"],
                    thermal_equipment["Info"].dict["Id"],
                )
            )
        return

    if print_heading:
        print(
            "{} {} Leak Detection".format(
                thermal_equipment["Info"].dict["EquipmentType"],
                thermal_equipment["Info"].dict["Id"],
            )
        )
        print("")

    detector_line_format = "  {:16s} | {:30s} | {:12s}"

    # Print info from leak detector groups
    grouped_detectors = []
    if "LeakDetectorGroups" in thermal_equipment["LeakDetection"].dict:
        for i, group in enumerate(thermal_equipment["LeakDetection"].dict["LeakDetectorGroups"]):
            group_name = "Leak Detection Group {}".format(i)
            if "GroupName" in group:
                group_name = group["GroupName"]
            prop_val = group.get("Status", {}).get("Health", "OK")
            if "HumidityPercent" in group:
                prop_val = prop_val + ", Humidity: {} %".format(
                    str(group.get("HumidityPercent", {}).get("Reading", "---"))
                )
            print("{}: {}".format(group_name, prop_val))
            print(detector_line_format.format("Detector Id", "Name", "State"))
            if "Detectors" in group:
                for j, detector in enumerate(group["Detectors"]):
                    detector_id = detector.get("DataSourceUri", "").split("/")[-1]
                    if "DataSourceUri" in detector:
                        grouped_detectors.append(detector["DataSourceUri"])
                    print(
                        detector_line_format.format(
                            detector_id, detector.get("DeviceName", ""), detector.get("DetectorState", "N/A")
                        )
                    )
            print("")

    # Find any detectors that do not belong to groups and print their info
    non_grouped_detectors = []
    if thermal_equipment["LeakDetectors"]:
        for detector in thermal_equipment["LeakDetectors"]:
            if detector.dict["@odata.id"] not in grouped_detectors:
                non_grouped_detectors.append(detector)
    if non_grouped_detectors:
        print("Other Leak Detectors")
        print(detector_line_format.format("Detector Id", "Name", "State"))
        for detector in non_grouped_detectors:
            print(
                detector_line_format.format(
                    detector.dict["Id"], detector.dict["Name"], detector.dict.get("DetectorState", "N/A")
                )
            )
        print("")


def get_coolant_string(coolant):
    """
    Builds a string based on the contents of the common coolant object

    Args:
        coolant: The coolant object to parse

    Returns:
        A string of the coolant information
    """

    coolant_info = []
    if "CoolantType" in coolant:
        coolant_info.append(str(coolant["CoolantType"]))
    if "AdditivePercent" in coolant and "AdditiveName" in coolant:
        coolant_info.append("{} % {}".format(coolant["AdditivePercent"], coolant["AdditiveName"]))
    if "SpecificHeatkJoulesPerKgK" in coolant:
        coolant_info.append("Specific Heat: {} kJ/kg/K".format(coolant["SpecificHeatkJoulesPerKgK"]))
    if "DensityKgPerCubicMeter" in coolant:
        coolant_info.append("Density: {} kg/m3".format(coolant["DensityKgPerCubicMeter"]))
    return ", ".join(coolant_info)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

from .accounts import get_users
from .accounts import print_users
from .accounts import add_user
from .accounts import delete_user
from .accounts import modify_user
from .assembly import get_assembly
from .assembly import print_assembly
from .assembly import download_assembly
from .assembly import upload_assembly
from .certificates import get_all_certificates
from .certificates import print_certificates
from .certificates import get_generate_csr_info
from .certificates import generate_csr
from .certificates import install_certificate
from .certificates import delete_certificate
from .event_service import get_event_service
from .event_service import print_event_service
from .event_service import get_event_subscriptions
from .event_service import print_event_subscriptions
from .event_service import create_event_subscription
from .event_service import delete_event_subscription
from .inventory import get_system_inventory
from .inventory import print_system_inventory
from .inventory import write_system_inventory
from .licenses import get_licenses
from .licenses import print_licenses
from .licenses import install_license
from .licenses import delete_license
from .logs import log_container
from .logs import diagnostic_data_types
from .logs import get_log_service_ids
from .logs import get_log_service
from .logs import get_log_entries
from .logs import print_log_entries
from .logs import clear_log_entries
from .logs import collect_diagnostic_data
from .logs import download_diagnostic_data
from .managers import get_manager_ids
from .managers import get_manager
from .managers import set_manager
from .managers import print_manager
from .managers import get_manager_reset_info
from .managers import manager_reset
from .managers import get_manager_reset_to_defaults_info
from .managers import manager_reset_to_defaults
from .managers import get_manager_network_protocol
from .managers import set_manager_network_protocol
from .managers import print_manager_network_protocol
from .managers import get_manager_ethernet_interface_ids
from .managers import get_manager_ethernet_interface
from .managers import set_manager_ethernet_interface
from .managers import print_manager_ethernet_interface
from .messages import print_error_payload
from .messages import verify_response
from .power_equipment import power_equipment_types
from .power_equipment import power_equipment_electrical_types
from .power_equipment import get_power_equipment_ids
from .power_equipment import get_power_equipment
from .power_equipment import print_power_equipment
from .power_equipment import get_power_equipment_summary
from .power_equipment import print_power_equipment_summary
from .power_equipment import get_power_equipment_electrical
from .power_equipment import print_power_equipment_electrical
from .power_equipment import print_power_equipment_electrical_summary
from .resets import reset_types
from .resets import reset_to_defaults_types
from .sensors import get_sensors
from .sensors import print_sensors
from .systems import get_system_ids
from .systems import get_system
from .systems import get_system_boot
from .systems import set_system_boot
from .systems import print_system_boot
from .systems import get_system_reset_info
from .systems import system_reset
from .systems import get_virtual_media
from .systems import print_virtual_media
from .systems import insert_virtual_media
from .systems import eject_virtual_media
from .systems import get_system_bios
from .systems import set_system_bios
from .systems import print_system_bios
from .systems import reset_system_bios
from .tasks import poll_task_monitor
from .thermal_equipment import thermal_equipment_types
from .thermal_equipment import thermal_equipment_component_types
from .thermal_equipment import get_thermal_equipment_ids
from .thermal_equipment import get_thermal_equipment
from .thermal_equipment import print_thermal_equipment
from .thermal_equipment import get_thermal_equipment_summary
from .thermal_equipment import print_thermal_equipment_summary
from .thermal_equipment import get_thermal_equipment_component
from .thermal_equipment import print_thermal_equipment_component
from .thermal_equipment import print_thermal_equipment_component_summary
from .thermal_equipment import print_thermal_equipment_leak_detector_summary
from .update import operation_apply_times
from .update import get_update_service
from .update import get_simple_update_info
from .update import simple_update
from .update import multipart_push_update
from .update import get_firmware_inventory
from .update import print_software_inventory
from .misc import logout, print_password_change_required_and_logout

from . import config

__all__ = [
    "get_users",
    "print_users",
    "add_user",
    "delete_user",
    "modify_user",
    "get_assembly",
    "print_assembly",
    "download_assembly",
    "upload_assembly",
    "get_all_certificates",
    "print_certificates",
    "get_generate_csr_info",
    "generate_csr",
    "install_certificate",
    "delete_certificate",
    "get_event_service",
    "print_event_service",
    "get_event_subscriptions",
    "print_event_subscriptions",
    "create_event_subscription",
    "delete_event_subscription",
    "get_system_inventory",
    "print_system_inventory",
    "write_system_inventory",
    "get_licenses",
    "print_licenses",
    "install_license",
    "delete_license",
    "log_container",
    "diagnostic_data_types",
    "get_log_service_ids",
    "get_log_service",
    "get_log_entries",
    "print_log_entries",
    "clear_log_entries",
    "collect_diagnostic_data",
    "download_diagnostic_data",
    "get_manager_ids",
    "get_manager",
    "set_manager",
    "print_manager",
    "get_manager_reset_info",
    "manager_reset",
    "get_manager_reset_to_defaults_info",
    "manager_reset_to_defaults",
    "get_manager_network_protocol",
    "set_manager_network_protocol",
    "print_manager_network_protocol",
    "get_manager_ethernet_interface_ids",
    "get_manager_ethernet_interface",
    "set_manager_ethernet_interface",
    "print_manager_ethernet_interface",
    "print_error_payload",
    "verify_response",
    "power_equipment_types",
    "power_equipment_electrical_types",
    "get_power_equipment_ids",
    "get_power_equipment",
    "print_power_equipment",
    "get_power_equipment_summary",
    "print_power_equipment_summary",
    "get_power_equipment_electrical",
    "print_power_equipment_electrical",
    "print_power_equipment_electrical_summary",
    "reset_types",
    "reset_to_defaults_types",
    "get_sensors",
    "print_sensors",
    "get_system_ids",
    "get_system",
    "get_system_boot",
    "set_system_boot",
    "print_system_boot",
    "get_system_reset_info",
    "system_reset",
    "get_virtual_media",
    "print_virtual_media",
    "insert_virtual_media",
    "eject_virtual_media",
    "get_system_bios",
    "set_system_bios",
    "print_system_bios",
    "reset_system_bios",
    "poll_task_monitor",
    "thermal_equipment_types",
    "thermal_equipment_component_types",
    "get_thermal_equipment_ids",
    "get_thermal_equipment",
    "print_thermal_equipment",
    "get_thermal_equipment_summary",
    "print_thermal_equipment_summary",
    "get_thermal_equipment_component",
    "print_thermal_equipment_component",
    "print_thermal_equipment_component_summary",
    "print_thermal_equipment_leak_detector_summary",
    "operation_apply_times",
    "get_update_service",
    "get_simple_update_info",
    "simple_update",
    "multipart_push_update",
    "get_firmware_inventory",
    "print_software_inventory",
    "logout",
    "print_password_change_required_and_logout",
    "config",
]



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Logs

File : rf_logs.py

Brief : This script uses the redfish_utilities module to manage logs
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage logs on a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument(
    "--manager", "-m", type=str, nargs="?", default=False, help="The ID of the manager containing the log service"
)
argget.add_argument(
    "--system", "-s", type=str, nargs="?", default=False, help="The ID of the system containing the log service"
)
argget.add_argument(
    "--chassis", "-c", type=str, nargs="?", default=False, help="The ID of the chassis containing the log service"
)
argget.add_argument("--log", "-l", type=str, help="The ID of the resource containing the log service")
argget.add_argument(
    "--details", "-details", action="store_true", help="Indicates details to be shown for each log entry"
)
argget.add_argument("--clear", "-clear", action="store_true", help="Indicates if the log should be cleared")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

# Determine the target log service based on the inputs
# Effectively if the user gives multiple targets, some will be ignored
container_type = redfish_utilities.log_container.MANAGER
container_id = None
if args.manager is not False:
    container_type = redfish_utilities.log_container.MANAGER
    container_id = args.manager
elif args.system is not False:
    container_type = redfish_utilities.log_container.SYSTEM
    container_id = args.system
elif args.chassis is not False:
    container_type = redfish_utilities.log_container.CHASSIS
    container_id = args.chassis

if args.debug:
    log_file = "rf_logs-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_logs Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=30, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    # Either clear the logs or get/print the logs
    if args.clear:
        # Clear log was requested
        print("Clearing the log...")
        response = redfish_utilities.clear_log_entries(redfish_obj, container_type, container_id, args.log)
        response = redfish_utilities.poll_task_monitor(redfish_obj, response)
        redfish_utilities.verify_response(response)
    else:
        # Print log was requested
        log_entries = redfish_utilities.get_log_entries(redfish_obj, container_type, container_id, args.log)
        try:
            from signal import signal, SIGPIPE, SIG_DFL

            signal(SIGPIPE, SIG_DFL)
        except Exception:
            # Windows does not support SIGPIPE; no need to modify the handling
            pass
        redfish_utilities.print_log_entries(log_entries, args.details)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Manager Configuration

File : rf_manager_config.py

Brief : This script uses the redfish_utilities module to manage managers
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage managers in a service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--manager", "-m", type=str, help="The ID of the manager to target")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
info_argget = subparsers.add_parser("info", help="Displays information about a manager")
reset_argget = subparsers.add_parser("reset", help="Resets a manager")
reset_argget.add_argument(
    "--type", "-t", type=str, help="The type of power/reset operation to perform", choices=redfish_utilities.reset_types
)
reset_argget.add_argument(
    "--info", "-info", action="store_true", help="Indicates if reset information should be reported"
)
getnet_argget = subparsers.add_parser("getnet", help="Displays information about an Ethernet interface")
getnet_argget.add_argument("--id", "-i", type=str, help="The identifier of the Ethernet interface to display")
setnet_argget = subparsers.add_parser("setnet", help="Configures an Ethernet interface")
setnet_argget.add_argument("--id", "-i", type=str, help="The identifier of the Ethernet interface to configure")
setnet_argget.add_argument("--ipv4address", "-ipv4address", type=str, help="The static IPv4 address to set")
setnet_argget.add_argument("--ipv4netmask", "-ipv4netmask", type=str, help="The static IPv4 subnet mask to set")
setnet_argget.add_argument("--ipv4gateway", "-ipv4gateway", type=str, help="The static IPv4 gateway to set")
setnet_argget.add_argument(
    "--dhcpv4", "-dhcpv4", type=str, help="The DHCPv4 configuration to set", choices=["On", "Off"]
)
setnet_argget.add_argument(
    "--ipv6addresses", "-ipv6addresses", type=str, nargs="+", help="The static IPv6 addresses to set with prefix length"
)
setnet_argget.add_argument(
    "--ipv6gateways",
    "-ipv6gateways",
    type=str,
    nargs="+",
    help="The static IPv6 default gateways to set with prefix length",
)
setnet_argget.add_argument(
    "--dhcpv6",
    "-dhcpv6",
    type=str,
    help="The DHCPv6 configuration to set",
    choices=["Stateful", "Stateless", "Disabled", "Enabled"],
)
setnet_argget.add_argument(
    "--vlan", "-vlan", type=str, help="The VLAN enabled configuration to set", choices=["On", "Off"]
)
setnet_argget.add_argument("--vlanid", "-vlanid", type=int, help="The VLAN ID to set")
setnet_argget.add_argument("--vlanpriority", "-vlanpriority", type=int, help="The VLAN priority to set")
reset_to_defaults_argget = subparsers.add_parser("resettodefaults", help="Resets a manager to default settings")
reset_to_defaults_argget.add_argument(
    "--type",
    "-t",
    type=str,
    help="The type of reset-to-defaults operation to perform",
    choices=redfish_utilities.reset_to_defaults_types,
)
reset_to_defaults_argget.add_argument(
    "--info", "-info", action="store_true", help="Indicates if reset-to-defaults information should be reported"
)
set_time_argget = subparsers.add_parser("settime", help="Sets the date-time on a manager")
set_time_argget.add_argument("--datetime", "-dt", type=str, help="The date-time value to set")
set_time_argget.add_argument("--offset", "-o", type=str, help="The date-time offset value to set")
getprotocol_argget = subparsers.add_parser("getprotocol", help="Displays network protocol information about a manager")
setprotocol_argget = subparsers.add_parser("setprotocol", help="Configures network protocol settings on a manager")
setprotocol_argget.add_argument("--protocol", "-prot", type=str, required=True, help="The protocol to set")
setprotocol_argget.add_argument("--enable", "-en", action="store_true", help="Enable the selected protocol")
setprotocol_argget.add_argument("--disable", "-dis", action="store_true", help="Disable the selected protocol")
setprotocol_argget.add_argument("--port", "-port", type=int, help="The port number to assign the protocol")
args = argget.parse_args()

if args.debug:
    log_file = "rf_manager_config-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_manager_config Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.command == "reset":
        if args.info:
            reset_uri, reset_parameters = redfish_utilities.get_manager_reset_info(redfish_obj, args.manager)
            printed_reset_types = False
            for param in reset_parameters:
                if param["Name"] == "ResetType" and "AllowableValues" in param:
                    print("Supported reset types: {}".format(", ".join(param["AllowableValues"])))
                    printed_reset_types = True
            if not printed_reset_types:
                print("No reset information found")
        else:
            print("Resetting the manager...")
            response = redfish_utilities.manager_reset(redfish_obj, args.manager, args.type)
            response = redfish_utilities.poll_task_monitor(redfish_obj, response)
            redfish_utilities.verify_response(response)
    elif args.command == "resettodefaults":
        if args.info:
            reset_uri, reset_parameters = redfish_utilities.get_manager_reset_to_defaults_info(
                redfish_obj, args.manager
            )
            printed_reset_types = False
            for param in reset_parameters:
                if param["Name"] == "ResetType" and "AllowableValues" in param:
                    print("Supported reset types: {}".format(", ".join(param["AllowableValues"])))
                    printed_reset_types = True
            if not printed_reset_types:
                print("No reset information found")
        else:
            print("Resetting the manager to defaults...")
            response = redfish_utilities.manager_reset_to_defaults(redfish_obj, args.manager, args.type)
            response = redfish_utilities.poll_task_monitor(redfish_obj, response)
            redfish_utilities.verify_response(response)
    elif args.command == "settime":
        redfish_utilities.set_manager(redfish_obj, args.manager, args.datetime, args.offset)
    elif args.command == "getnet":
        interface = redfish_utilities.get_manager_ethernet_interface(redfish_obj, args.manager, args.id)
        redfish_utilities.print_manager_ethernet_interface(interface)
    elif args.command == "setnet":
        vlan = {}
        if args.vlan == "On":
            vlan["VLANEnable"] = True
        elif args.vlan == "Off":
            vlan["VLANEnable"] = False
        if args.vlanid is not None:
            vlan["VLANId"] = args.vlanid
        if args.vlanpriority is not None:
            vlan["VLANPriority"] = args.vlanpriority
        if len(vlan) == 0:
            vlan = None
        ipv4_addresses = [{}]
        if args.ipv4address is not None:
            ipv4_addresses[0]["Address"] = args.ipv4address
        if args.ipv4netmask is not None:
            ipv4_addresses[0]["SubnetMask"] = args.ipv4netmask
        if args.ipv4gateway is not None:
            ipv4_addresses[0]["Gateway"] = args.ipv4gateway
        if len(ipv4_addresses[0]) == 0:
            ipv4_addresses = None
        dhcpv4 = None
        if args.dhcpv4 == "On":
            dhcpv4 = {"DHCPEnabled": True}
        elif args.dhcpv4 == "Off":
            dhcpv4 = {"DHCPEnabled": False}
        dhcpv6 = None
        ipv6_addresses = None
        if args.ipv6addresses is not None:
            ipv6_addresses = []
            for address in args.ipv6addresses:
                if address == "None":
                    ipv6_addresses.append(None)
                elif address == "{}":
                    ipv6_addresses.append({})
                else:
                    ipv6_addresses.append(
                        {"Address": address.split("/")[0], "PrefixLength": int(address.split("/")[1])}
                    )
        ipv6_gateways = None
        if args.ipv6gateways is not None:
            ipv6_gateways = []
            for address in args.ipv6gateways:
                if address == "None":
                    ipv6_gateways.append(None)
                elif address == "{}":
                    ipv6_gateways.append({})
                else:
                    ipv6_gateways.append({"Address": address.split("/")[0], "PrefixLength": int(address.split("/")[1])})
        if args.dhcpv6 is not None:
            dhcpv6 = {"OperatingMode": args.dhcpv6}
        redfish_utilities.set_manager_ethernet_interface(
            redfish_obj, args.manager, args.id, vlan, ipv4_addresses, dhcpv4, ipv6_addresses, ipv6_gateways, dhcpv6
        )
    elif args.command == "getprotocol":
        network_protocol = redfish_utilities.get_manager_network_protocol(redfish_obj, args.manager)
        redfish_utilities.print_manager_network_protocol(network_protocol)
    elif args.command == "setprotocol":
        network_protocol_setting = {args.protocol: {}}
        if args.enable:
            network_protocol_setting[args.protocol]["ProtocolEnabled"] = True
        elif args.disable:
            network_protocol_setting[args.protocol]["ProtocolEnabled"] = False
        if args.port:
            network_protocol_setting[args.protocol]["Port"] = args.port
        print("Configuring {}...".format(args.protocol))
        redfish_utilities.set_manager_network_protocol(redfish_obj, args.manager, network_protocol_setting)
    else:
        manager = redfish_utilities.get_manager(redfish_obj, args.manager)
        redfish_utilities.print_manager(manager)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Event Service Module

File : event_service.py

Brief : This file contains the definitions and functionalities for managing
        accounts on a Redfish Service
"""

from .messages import verify_response


class RedfishEventServiceNotFoundError(Exception):
    """
    Raised when the event service cannot be found
    """

    pass


class RedfishEventSubscriptionNotFoundError(Exception):
    """
    Raised when the specified event subscription cannot be found
    """

    pass


def get_event_service(context):
    """
    Collects the event service information from a Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        An object containing information about the event service
    """

    # Get the service root to find the event service
    service_root = context.get("/redfish/v1/")
    if "EventService" not in service_root.dict:
        # No event service
        raise RedfishEventServiceNotFoundError("Service does not contain an event service")

    # Get the event service
    event_service = context.get(service_root.dict["EventService"]["@odata.id"])
    return event_service.dict


def print_event_service(service):
    """
    Prints the event service information from a Redfish service

    Args:
        service: The event service object
    """

    print("Service Info")

    # General status
    status = "Enabled"
    if "ServiceEnabled" in service:
        if not service["ServiceEnabled"]:
            status = "Disabled"
    if "Status" in service:
        if "State" in service["Status"]:
            status = service["Status"]["State"]
    print("  Status: {}".format(status))

    # Delivery retry policy
    retry_policy = ""
    if "DeliveryRetryAttempts" in service:
        retry_policy += "{} attempts, ".format(service["DeliveryRetryAttempts"])
    if "DeliveryRetryIntervalSeconds" in service:
        retry_policy += "{} second intervals, ".format(service["DeliveryRetryIntervalSeconds"])
    if len(retry_policy) > 2:
        retry_policy = retry_policy[:-2]
    else:
        retry_policy = "Unknown/Unspecified"
    print("  Delivery Retry Policy: {}".format(retry_policy))

    # Subscription methods
    print("  Event Types: {}".format(", ".join(service.get("EventTypesForSubscription", ["N/A"]))))
    print("  Event Formats: {}".format(", ".join(service.get("EventFormatTypes", ["N/A"]))))
    print("  Registries: {}".format(", ".join(service.get("RegistryPrefixes", ["N/A"]))))
    print("  Resource Types: {}".format(", ".join(service.get("ResourceTypes", ["N/A"]))))
    print("  Include Origin of Condition Supported: {}".format(service.get("IncludeOriginOfConditionSupported", False)))

    # SSE info
    print("  SSE URI: {}".format(service.get("ServerSentEventUri", "Not supported")))
    sse_filters = ""
    for filter in service.get("SSEFilterPropertiesSupported", {}):
        if filter == "EventType":
            # This style is deprecated
            continue
        if service["SSEFilterPropertiesSupported"][filter]:
            sse_filters += "{}, ".format(filter)
    if len(sse_filters) > 2:
        sse_filters = sse_filters[:-2]
    else:
        sse_filters = "Unknown/Unspecified"
    print("  SSE Filter Parameters: {}".format(sse_filters))


def get_event_subscriptions(context):
    """
    Collects the event subscription information from a Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list containing all event subscriptions
    """

    # Get the event service
    event_service = get_event_service(context)
    if "Subscriptions" not in event_service:
        # No subscriptions
        raise RedfishEventServiceNotFoundError("Service does not contain a subscription collection")

    # Get each of the event subscriptions
    subscriptions = []
    subscription_col = context.get(event_service["Subscriptions"]["@odata.id"])
    for subscription_member in subscription_col.dict["Members"]:
        subscription = context.get(subscription_member["@odata.id"])
        subscriptions.append(subscription.dict)
    return subscriptions


def print_event_subscriptions(subscriptions):
    """
    Prints the event subscription information from a Redfish service

    Args:
        subscriptions: An array of event subscription objects
    """

    subscription_line_format = "  {:36s} | {}: {}"

    print("Subscription Info")

    if len(subscriptions) == 0:
        print("  No subscriptions")
        return

    for subscription in subscriptions:
        print(
            subscription_line_format.format(subscription["Id"], "Destination", subscription.get("Destination", "N/A"))
        )
        print(subscription_line_format.format("", "State", subscription.get("Status", {}).get("State", "Enabled")))
        if "Context" in subscription:
            print(subscription_line_format.format("", "Context", subscription["Context"]))
        print(subscription_line_format.format("", "Event Format", subscription.get("EventFormatType", "Event")))
        if "EventTypes" in subscription:
            print(subscription_line_format.format("", "Event Types", ", ".join(subscription["EventTypes"])))
        if "RegistryPrefixes" in subscription:
            print(subscription_line_format.format("", "Registries", ", ".join(subscription["RegistryPrefixes"])))
        if "ResourceTypes" in subscription:
            print(subscription_line_format.format("", "Resource Types", ", ".join(subscription["ResourceTypes"])))


def create_event_subscription(
    context,
    destination,
    format=None,
    client_context=None,
    expand=None,
    resource_types=None,
    registries=None,
    message_ids=None,
    origins=None,
    subordinate_resources=None,
    event_types=None,
):
    """
    Creates an event subscription

    Args:
        context: The Redfish client object with an open session
        destination: The event subscription destination
        format: The format of the event payloads
        client_context: The client-provided context string
        expand: Indicates if the OriginOfCondition is to be expanded in event payloads
        resource_types: The resource types for the subscription
        registries: The registries for the subscription
        message_ids: The message IDs for the subscription
        origins: The origins for the subscription
        subordinate_resources: Indicates if subordinate resources to those referenced by 'origins' will also be monitored
        event_types: The event types for the subscription; this method for subscriptions has been deprecated for other controls

    Returns:
        The response of the POST
    """

    # Get the event service
    event_service = get_event_service(context)
    if "Subscriptions" not in event_service:
        # No subscriptions
        raise RedfishEventServiceNotFoundError("Service does not contain a subscription collection")

    # Form the POST request
    payload = {"Destination": destination, "Protocol": "Redfish"}
    if format is not None:
        payload["EventFormatType"] = format
    if client_context is not None:
        payload["Context"] = client_context
    if expand is not None:
        payload["IncludeOriginOfCondition"] = expand
    if resource_types is not None:
        payload["ResourceTypes"] = resource_types
    if registries is not None:
        payload["RegistryPrefixes"] = registries
    if message_ids is not None:
        payload["MessageIds"] = message_ids
    if origins is not None:
        payload["OriginResources"] = origins
    if subordinate_resources is not None:
        payload["SubordinateResources"] = subordinate_resources
    if event_types is not None:
        payload["EventTypes"] = event_types

    # Create the subscription
    response = context.post(event_service["Subscriptions"]["@odata.id"], body=payload)
    verify_response(response)
    return response


def delete_event_subscription(context, id):
    """
    Deletes an event subscription

    Args:
        context: The Redfish client object with an open session
        id: The identifier for the subscription

    Returns:
        The response of the DELETE
    """

    # Get the current subscriptions
    subscriptions = get_event_subscriptions(context)

    # Find the matching subscription and delete it
    for subscription in subscriptions:
        if subscription["Id"] == id:
            response = context.delete(subscription["@odata.id"])
            verify_response(response)
            return response

    # No matches found
    raise RedfishEventSubscriptionNotFoundError("Service does not contain an event subscription called {}".format(id))



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Thermal Equipment

File : rf_thermal_equipment.py

Brief : This script uses the redfish_utilities module to manage thermal equipment
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage thermal equipment")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
list_argget = subparsers.add_parser("list", help="Displays a list of the available thermal equipment")
status_argget = subparsers.add_parser("status", help="Displays the status of an instance of thermal equipment")
status_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.thermal_equipment_types,
    required=True,
    help="The type of thermal equipment to get",
    choices=redfish_utilities.thermal_equipment_types,
)
status_argget.add_argument("--equipment", "-te", type=str, help="The identifier of the thermal equipment to get")
primary_info_argget = subparsers.add_parser(
    "primaryinfo", help="Displays the status of a primary coolant connector for an instance of thermal equipment"
)
primary_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.thermal_equipment_types,
    required=True,
    help="The type of thermal equipment to get",
    choices=redfish_utilities.thermal_equipment_types,
)
primary_info_argget.add_argument("--equipment", "-te", type=str, help="The identifier of the thermal equipment to get")
primary_info_argget.add_argument(
    "--primary", "-pr", type=str, help="The identifier of the primary coolant connector to get"
)
secondary_info_argget = subparsers.add_parser(
    "secondaryinfo", help="Displays the status of a secondary coolant connector for an instance of thermal equipment"
)
secondary_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.thermal_equipment_types,
    required=True,
    help="The type of thermal equipment to get",
    choices=redfish_utilities.thermal_equipment_types,
)
secondary_info_argget.add_argument(
    "--equipment", "-te", type=str, help="The identifier of the thermal equipment to get"
)
secondary_info_argget.add_argument(
    "--secondary", "-sec", type=str, help="The identifier of the secondary coolant connector to get"
)
pump_info_argget = subparsers.add_parser(
    "pumpinfo", help="Displays the status of a pump for an instance of thermal equipment"
)
pump_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.thermal_equipment_types,
    required=True,
    help="The type of thermal equipment to get",
    choices=redfish_utilities.thermal_equipment_types,
)
pump_info_argget.add_argument("--equipment", "-te", type=str, help="The identifier of the thermal equipment to get")
pump_info_argget.add_argument("--pump", "-pu", type=str, help="The identifier of the pump to get")
filter_info_argget = subparsers.add_parser(
    "filterinfo", help="Displays the status of a filter for an instance of thermal equipment"
)
filter_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.thermal_equipment_types,
    required=True,
    help="The type of thermal equipment to get",
    choices=redfish_utilities.thermal_equipment_types,
)
filter_info_argget.add_argument("--equipment", "-te", type=str, help="The identifier of the thermal equipment to get")
filter_info_argget.add_argument("--filter", "-fil", type=str, help="The identifier of the filter to get")
reservoir_info_argget = subparsers.add_parser(
    "reservoirinfo", help="Displays the status of a reservoir for an instance of thermal equipment"
)
reservoir_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.thermal_equipment_types,
    required=True,
    help="The type of thermal equipment to get",
    choices=redfish_utilities.thermal_equipment_types,
)
reservoir_info_argget.add_argument(
    "--equipment", "-te", type=str, help="The identifier of the thermal equipment to get"
)
reservoir_info_argget.add_argument("--reservoir", "-res", type=str, help="The identifier of the reservoir to get")
leak_detectors_argget = subparsers.add_parser(
    "leakdetectors", help="Displays the leak detector summary of an instance of thermal equipment"
)
leak_detectors_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.thermal_equipment_types,
    required=True,
    help="The type of thermal equipment to get",
    choices=redfish_utilities.thermal_equipment_types,
)
leak_detectors_argget.add_argument(
    "--equipment", "-te", type=str, help="The identifier of the thermal equipment to get"
)
args = argget.parse_args()

if args.debug:
    log_file = "rf_thermal_equipment-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_thermal_equipment Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.command == "status":
        thermal_equipment = redfish_utilities.get_thermal_equipment(
            redfish_obj,
            args.type,
            args.equipment,
            get_metrics=True,
            get_primary_connectors=True,
            get_secondary_connectors=True,
            get_pumps=True,
            get_filters=True,
            get_reservoirs=True,
            get_leak_detection=True,
        )
        redfish_utilities.print_thermal_equipment(thermal_equipment)
        component_types = [
            redfish_utilities.thermal_equipment_component_types.PRIMARY_CONNECTOR,
            redfish_utilities.thermal_equipment_component_types.SECONDARY_CONNECTOR,
            redfish_utilities.thermal_equipment_component_types.PUMP,
            redfish_utilities.thermal_equipment_component_types.FILTER,
            redfish_utilities.thermal_equipment_component_types.RESERVOIR,
        ]
        for component_type in component_types:
            redfish_utilities.print_thermal_equipment_component_summary(thermal_equipment, component_type, False)
    elif args.command == "primaryinfo":
        primary = redfish_utilities.get_thermal_equipment_component(
            redfish_obj,
            args.type,
            redfish_utilities.thermal_equipment_component_types.PRIMARY_CONNECTOR,
            args.equipment,
            args.primary,
        )
        redfish_utilities.print_thermal_equipment_component(primary)
    elif args.command == "secondaryinfo":
        secondary = redfish_utilities.get_thermal_equipment_component(
            redfish_obj,
            args.type,
            redfish_utilities.thermal_equipment_component_types.SECONDARY_CONNECTOR,
            args.equipment,
            args.secondary,
        )
        redfish_utilities.print_thermal_equipment_component(secondary)
    elif args.command == "pumpinfo":
        pump = redfish_utilities.get_thermal_equipment_component(
            redfish_obj, args.type, redfish_utilities.thermal_equipment_component_types.PUMP, args.equipment, args.pump
        )
        redfish_utilities.print_thermal_equipment_component(pump)
    elif args.command == "filterinfo":
        filter = redfish_utilities.get_thermal_equipment_component(
            redfish_obj,
            args.type,
            redfish_utilities.thermal_equipment_component_types.FILTER,
            args.equipment,
            args.filter,
        )
        redfish_utilities.print_thermal_equipment_component(filter)
    elif args.command == "reservoirinfo":
        reservoir = redfish_utilities.get_thermal_equipment_component(
            redfish_obj,
            args.type,
            redfish_utilities.thermal_equipment_component_types.RESERVOIR,
            args.equipment,
            args.reservoir,
        )
        redfish_utilities.print_thermal_equipment_component(reservoir)
    elif args.command == "leakdetectors":
        thermal_equipment = redfish_utilities.get_thermal_equipment(
            redfish_obj,
            args.type,
            args.equipment,
            get_leak_detection=True,
            get_leak_detectors=True,
        )
        redfish_utilities.print_thermal_equipment_leak_detector_summary(thermal_equipment, True)
    else:
        thermal_equipment = redfish_utilities.get_thermal_equipment_summary(redfish_obj)
        redfish_utilities.print_thermal_equipment_summary(thermal_equipment)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Certificates

File : rf_certificates.py

Brief : This script uses the redfish_utilities module to manage certificates
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage certificates on a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
info_argget = subparsers.add_parser("info", help="Displays information about the certificates installed on the service")
info_argget.add_argument(
    "--details",
    "-details",
    action="store_true",
    help="Indicates if the full details of each certificate should be shown",
)
csrinfo_argget = subparsers.add_parser(
    "csrinfo", help="Displays information about options supported for generating certificate signing requests"
)
csr_argget = subparsers.add_parser("csr", help="Generates a certificate signing request")
csr_argget.add_argument(
    "--certificatecollection",
    "-col",
    type=str,
    required=True,
    help="The URI of the certificate collection where the signed certificate will be installed",
)
csr_argget.add_argument(
    "--commonname", "-cn", type=str, required=True, help="The common name of the component to secure"
)
csr_argget.add_argument(
    "--organization", "-o", type=str, required=True, help="The name of the unit in the organization making the request"
)
csr_argget.add_argument(
    "--organizationalunit",
    "-ou",
    type=str,
    required=True,
    help="The name of the unit in the organization making the request",
)
csr_argget.add_argument(
    "--city", "-l", type=str, required=True, help="The city or locality of the organization making the request"
)
csr_argget.add_argument(
    "--state",
    "-st",
    type=str,
    required=True,
    help="The state, province, or region of the organization making the request",
)
csr_argget.add_argument(
    "--country",
    "-c",
    type=str,
    required=True,
    help="The two-letter country code of the organization making the request",
)
csr_argget.add_argument(
    "--email", "-email", type=str, help="The email address of the contact within the organization making the request"
)
csr_argget.add_argument("--keyalg", "-alg", type=str, help="The type of key-pair for use with signing algorithms")
csr_argget.add_argument(
    "--keylen", "-len", type=int, help="The length of the key, in bits, if the key pair algorithm supports key size"
)
csr_argget.add_argument(
    "--keycurve", "-curve", type=str, help="The curve ID to use with the key if the key pair algorithm supports curves"
)
csr_argget.add_argument(
    "--out", "-out", type=str, help="The file, with optional path, to save the certificate signing request"
)
install_argget = subparsers.add_parser("install", help="Installs a certificate on the service")
install_argget.add_argument(
    "--destination",
    "-dest",
    type=str,
    required=True,
    help="The installation URI of the certificate; either a certificate collection to insert, or an existing certificate to replace",
)
install_argget.add_argument(
    "--certificate", "-cert", type=str, required=True, help="The file, and optional path, of the certificate to install"
)
install_argget.add_argument(
    "--key", "-key", type=str, help="The file, and optional path, of the private key for the certificate to install"
)
delete_argget = subparsers.add_parser("delete", help="Deletes a certificate on the service")
delete_argget.add_argument(
    "--certificate", "-cert", type=str, required=True, help="The URI of the certificate to delete"
)
args = argget.parse_args()

if args.debug:
    log_file = "rf_certificates-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_certificates Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.command == "csrinfo":
        csr_uri, csr_parameters = redfish_utilities.get_generate_csr_info(redfish_obj)
        if csr_parameters is None:
            print("No generate CSR parameter information found")
        else:
            print("Generate CSR parameters:")
            for param in csr_parameters:
                required = ""
                if param.get("Required"):
                    required = " (required)"
                allowable_values = ""
                if param.get("AllowableValues"):
                    allowable_values = ": " + ", ".join(param["AllowableValues"])
                if param.get("AllowableNumbers"):
                    allowable_values = ": " + ", ".join(param["AllowableNumbers"])
                if param.get("AllowablePattern"):
                    allowable_values = ": " + param["AllowablePattern"]
                print("  {}{}{}".format(param["Name"], required, allowable_values))
    elif args.command == "csr":
        print("Generating cerficiate signing request...")
        response = redfish_utilities.generate_csr(
            redfish_obj,
            args.commonname,
            args.organization,
            args.organizationalunit,
            args.city,
            args.state,
            args.country,
            args.certificatecollection,
            args.email,
            args.keyalg,
            args.keylen,
            args.keycurve,
        )
        response = redfish_utilities.poll_task_monitor(redfish_obj, response)
        redfish_utilities.verify_response(response)
        print("")
        print(response.dict["CSRString"])
        print("")
        if args.out:
            with open(args.out, "w") as file:
                file.write(response.dict["CSRString"])
    elif args.command == "install":
        print("Installing {}...".format(args.certificate))
        response = redfish_utilities.install_certificate(redfish_obj, args.destination, args.certificate, args.key)
        response = redfish_utilities.poll_task_monitor(redfish_obj, response)
        redfish_utilities.verify_response(response)
    elif args.command == "delete":
        print("Deleting {}...".format(args.certificate))
        response = redfish_utilities.delete_certificate(redfish_obj, args.certificate)
        redfish_utilities.verify_response(response)
    else:
        certificates = redfish_utilities.get_all_certificates(redfish_obj)
        if args.command == "info":
            redfish_utilities.print_certificates(certificates, details=args.details)
        else:
            redfish_utilities.print_certificates(certificates)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2025 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish System Inventory

File : rf_sys_inventory.py

Brief : This script uses the redfish_utilities module to dump system inventory
        information
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to walk a Redfish service and list component information")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument(
    "--details", "-details", action="store_true", help="Indicates if the full details of each component should be shown"
)
argget.add_argument(
    "--noabsent", "-noabsent", action="store_true", help="Indicates if absent devices should be skipped"
)
argget.add_argument(
    "--write",
    "-w",
    nargs="?",
    const="Device_Inventory",
    type=str,
    help="Indicates if the inventory should be written to a spreadsheet and what the file name should be if given",
)
argget.add_argument(
    "--workaround",
    "-workaround",
    action="store_true",
    help="Indicates if workarounds should be attempted for non-conformant services",
    default=False,
)
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

if args.workaround:
    redfish_utilities.config.__workarounds__ = True

if args.debug:
    log_file = "rf_sys_inventory-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_sys_inventory Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    # Get and print the system inventory
    inventory = redfish_utilities.get_system_inventory(redfish_obj)
    redfish_utilities.print_system_inventory(inventory, args.details, args.noabsent)

    if args.write:
        redfish_utilities.write_system_inventory(inventory, args.write)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)
